@@ -18,9 +18,11 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
+#define _POSIX_C_SOURCE 199309L
+
 #include "config.h"
 
 #include <ctype.h>
@@ -58,8 +60,6 @@
 #define BARBIE_DATA_THUMBNAIL	1
 #define BARBIE_DATA_PICTURE	2
 
-#define PICTURE_SIZE(n1, n2, n3, n4)	(n1*(n2+n3)+n4)
-
 #define GP_MODULE "barbie"
 
 /* Some simple packet templates */
@@ -67,27 +67,26 @@ static const char packet_1[4]                = {0x02, 0x01, 0x01, 0x03};
 
 /* Utility Functions */
 static int
-barbie_read_response(GPPort *port, char *response, int size) {
+barbie_read_response(GPPort *port, unsigned char *response, unsigned int size) {
 	int x;
 	char ack = 0;
 
 	/* Read the ACK */
 	x=gp_port_read(port, &ack, 1);
 	if ((ack != ACK)||(x<0))
-		return (0);
+		return GP_ERROR_IO_READ;
 	/* Read the Response */
 	memset(response, 0, size);
-	x=gp_port_read(port,response, size);
-	return (x > 0);
+	return gp_port_read(port, (char*)response, size);
 }
 
 static
-int barbie_exchange (GPPort *port, char *cmd, int cmd_size, char *resp, int resp_size) {
+int barbie_exchange (GPPort *port, unsigned char *cmd, unsigned int cmd_size, unsigned char *resp, unsigned int resp_size) {
 	int count = 0;
 	while (count++ < 10) {
-		if (gp_port_write(port, cmd, cmd_size) < GP_OK)
+		if (gp_port_write(port, (char*)cmd, cmd_size) < GP_OK)
 			return (0);
-		if (barbie_read_response(port, resp, resp_size) != 1)
+		if (barbie_read_response(port, resp, resp_size) < GP_OK)
 			return (0);
 		/* if it's not busy, return */
 		if (resp[RESPONSE_BYTE] != '!')
@@ -100,7 +99,7 @@ int barbie_exchange (GPPort *port, char *cmd, int cmd_size, char *resp, int resp
 
 static int
 barbie_ping(GPPort *port) {
-	char cmd[4], resp[4];
+	unsigned char cmd[4], resp[4];
 
 	GP_DEBUG( "Pinging the camera...");
 
@@ -119,7 +118,7 @@ barbie_ping(GPPort *port) {
 
 static int
 barbie_file_count (GPPort *port) {
-        char cmd[4], resp[4];
+        unsigned char cmd[4], resp[4];
 
         GP_DEBUG ("Getting the number of pictures...");
 
@@ -132,117 +131,56 @@ barbie_file_count (GPPort *port) {
         return (resp[DATA1_BYTE]);
 }
 
-static char *
-barbie_read_data (GPPort *port, char *cmd, int cmd_size, int data_type, int *size) {
-	char c, resp[4];
-	int cols, visrows, blackrows, statusbytes;
-	int x, y, z;
-	char *s = NULL, *us = NULL, *rg = NULL, *t = NULL;
-	char ppmhead[64];
+static unsigned char *
+barbie_read_firmware(GPPort *port) {
 
-	if (barbie_exchange(port, cmd, cmd_size, resp, 4) != 1)
+	unsigned char cmd[4];
+	unsigned char resp[4];
+	unsigned int size;
+	char c;
+	unsigned char *s = NULL;
+
+	GP_DEBUG ("Getting Firmware...");
+
+	memcpy(cmd, packet_1, 4);
+	cmd[COMMAND_BYTE] = 'V';
+	cmd[DATA1_BYTE]   = '0';
+	if (barbie_exchange(port, cmd, 4, resp, 4) != 1)
 		return (0);
-	switch (data_type) {
-		case BARBIE_DATA_FIRMWARE:
-			GP_DEBUG ("Getting Firmware...");
-			/* we're getting the firmware revision */
-			*size = resp[2];
-			s = (char *)malloc(sizeof(char)*(*size));
-			memset(s, 0, *size);
-			s[0] = resp[3];
-			if (gp_port_read(port, &s[1], (*size)-1) < 0) {
-				free(s);
-				return (NULL);
-			}
-			break;
-		case BARBIE_DATA_PICTURE:
-			GP_DEBUG( "Getting Picture...");
-			/* we're getting a picture */
-			cols = (unsigned char)resp[2];
-			blackrows = (unsigned char)resp[3];
-			if (gp_port_read(port, &c, 1) < 0)
-				return (NULL);
-			visrows = (unsigned char)c;
-			if (gp_port_read(port, &c, 1) < 0)
-				return (NULL);
-			statusbytes = (unsigned char)c;
-			*size = PICTURE_SIZE(cols, blackrows, visrows, statusbytes);
-			sprintf(ppmhead, "P6\n# test.ppm\n%i %i\n255\n", cols-4, visrows);
-			us = (char *)malloc(sizeof(char)*(*size));
-			rg = (char *)malloc(sizeof(char)*(*size));
-			s  = (char *)malloc(sizeof(char)*cols*(blackrows+visrows)*3+strlen(ppmhead));
-			t  = (char *)malloc(sizeof(char)*(cols-4)*visrows*3+strlen(ppmhead));
-			memset(us, 0, *size);
-			memset(rg, 0, *size);
-			memset(s , 0, *size+strlen(ppmhead));
-			memset(t , 0, *size+strlen(ppmhead));
-			if (gp_port_read(port, us, *size)<0) {
-				free(us);
-				free(rg);
-				free(s);
-				return (NULL);
-			}
-			/* Unshuffle the data */
-			for (x=0; x<blackrows+visrows; x++) {
-				for (y=0; y<cols-4; y++) {
-					z = x*cols + y/2 + (y%2)*(cols/2+2);
-					rg[x*cols+(y^1)] = us[z];
-				}
-			}
-			free (us);
-			strcpy (t, ppmhead);
-			z = strlen(t);
-			/* Camera uses a Bayer CCD array:
-			 *		gb  gb   ...
-			 *		rg  rg   ...
-			 */
-			gp_bayer_decode (rg, cols, blackrows+visrows, s+z, BAYER_TILE_GBRG);
-			free (rg);
-			/* Shrink it a bit to avoid junk cols and black rows */
-			for (x=0; x<visrows; x++)
-				memcpy (t+z+x*(cols-4)*3,s+z+cols*(x+blackrows)*3,(cols-4)*3);
-			z+= visrows*(cols-4)*3;
-			*size = z;
-			memcpy(s,t,z);
-			free (t);
-			GP_DEBUG( "size=%i", *size);
-			break;
-		case BARBIE_DATA_THUMBNAIL:
-			break;
-		default:
-			break;
+	/* we're getting the firmware revision */
+	size = resp[2];
+	s = malloc(size);
+	memset(s, 0, size);
+	s[0] = resp[3];
+	if (gp_port_read(port, (char*)&s[1], size-1) < 0) {
+		free(s);
+		return (NULL);
 	}
 	/* read the footer */
 	if (gp_port_read(port, &c, 1) < 0) {
 		free(s);
 		return (0);
 	}
-	return(s);
+	return s;
 }
 
-static char *
-barbie_read_firmware(GPPort *port) {
-
-	char cmd[4];
-	int x;
-	
-	memcpy(cmd, packet_1, 4);
-	cmd[COMMAND_BYTE] = 'V';
-	cmd[DATA1_BYTE]   = '0';
-	return (barbie_read_data(port, cmd, 4, BARBIE_DATA_FIRMWARE, &x));
-}
-
-static char *
-barbie_read_picture(GPPort *port, int picture_number, int get_thumbnail, int *size) {
-
-	char cmd[4], resp[4];
+static int
+barbie_read_picture(GPPort *port, int picture_number, int get_thumbnail, CameraFile *file) {
+	unsigned char cmd[4], resp[4];
+	unsigned int size;
+	char c;
+	int cols, visrows, blackrows, statusbytes;
+	int x, y, z;
+	unsigned char *s = NULL, *us = NULL, *rg = NULL, *t = NULL;
+	char ppmhead[64];
+	unsigned char uc;
 
 	memcpy(cmd, packet_1, 4);
 	cmd[COMMAND_BYTE] = 'A';
 	cmd[DATA1_BYTE]   = picture_number;
 
 	if (barbie_exchange(port, cmd, 4, resp, 4) != 1)
-		return (NULL);
+		return GP_ERROR_IO_READ;
 
 	memcpy(cmd, packet_1, 4);
 	if (get_thumbnail)
@@ -251,7 +189,62 @@ barbie_read_picture(GPPort *port, int picture_number, int get_thumbnail, int *si
 		cmd[COMMAND_BYTE] = 'U';
 	cmd[DATA1_BYTE] = 0;
 
-	return (barbie_read_data(port, cmd, 4, BARBIE_DATA_PICTURE, size));
+	if (barbie_exchange(port, cmd, 4, resp, 4) != 1)
+		return (0);
+	GP_DEBUG( "Getting Picture...");
+	/* we're getting a picture */
+	cols = resp[2];
+	blackrows = resp[3];
+	if (gp_port_read(port, (char*)&uc, 1) < 0)
+		return GP_ERROR_IO_READ;
+	visrows = uc;
+	if (gp_port_read(port, (char*)&uc, 1) < 0)
+		return GP_ERROR_IO_READ;
+	statusbytes = uc;
+	size = cols*(blackrows+visrows)+statusbytes;
+	sprintf(ppmhead, "P6\n# test.ppm\n%i %i\n255\n", cols-4, visrows);
+	gp_file_append (file, ppmhead, strlen(ppmhead));
+
+	us = malloc(size);
+	rg = malloc(size);
+	s  = malloc(cols*(blackrows+visrows)*3);
+	t  = malloc((cols-4)*visrows*3);
+	memset(us, 0, size);
+	memset(rg, 0, size);
+	memset(s , 0, size);
+	memset(t , 0, size);
+	if (gp_port_read(port, (char*)us, size)<0) {
+		free(us);
+		free(rg);
+		free(s);
+		free(t);
+		return GP_ERROR_IO_READ;
+	}
+	/* Unshuffle the data */
+	for (x=0; x<blackrows+visrows; x++) {
+		for (y=0; y<cols-4; y++) {
+			z = x*cols + y/2 + (y%2)*(cols/2+2);
+			rg[x*cols+(y^1)] = us[z];
+		}
+	}
+	free (us);
+	/* Camera uses a Bayer CCD array:
+	 *		gb  gb   ...
+	 *		rg  rg   ...
+	 */
+	gp_bayer_decode (rg, cols, blackrows+visrows, s, BAYER_TILE_GBRG);
+	free (rg);
+	/* Shrink it a bit to avoid junk cols and black rows */
+	for (x=0; x<visrows; x++)
+		memcpy (t+x*(cols-4)*3,s+cols*(x+blackrows)*3,(cols-4)*3);
+	gp_file_append (file, (char*)t, visrows*(cols-4)*3);
+	free (s);
+	free (t);
+
+	/* read the footer */
+	if (gp_port_read(port, &c, 1) < 0)
+		return GP_ERROR_IO_READ;
+	return GP_OK;
 }
 
 
@@ -312,30 +305,24 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       GPContext *context)
 {
 	Camera *camera = user_data;
-        int size, num;
-        char *data;
+        int num;
 
         /* Retrieve the number of the photo on the camera */
         num = gp_filesystem_number (camera->fs, "/", filename, context);
 	if (num < GP_OK)
 		return num;
+        gp_file_set_mime_type (file, GP_MIME_PPM);
         switch (type) {
         case GP_FILE_TYPE_NORMAL:
-                data = barbie_read_picture (camera->port, num, 0, &size);
+                return barbie_read_picture (camera->port, num, 0, file);
                 break;
         case GP_FILE_TYPE_PREVIEW:
-                data = barbie_read_picture (camera->port, num, 1, &size);
+                return barbie_read_picture (camera->port, num, 1, file);
                 break;
         default:
-                return (GP_ERROR_NOT_SUPPORTED);
+                return GP_ERROR_NOT_SUPPORTED;
         }
-        if (!data)
-                return GP_ERROR;
-
-        gp_file_set_name (file, filename);
-        gp_file_set_mime_type (file, "image/ppm");
-        gp_file_set_data_and_size (file, data, size);
-        return GP_OK;
+        return GP_ERROR_NOT_SUPPORTED;
 }
 
 #if 0
@@ -368,12 +355,9 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
         char *firm;
 
         num = barbie_file_count (camera->port);
-        firm = barbie_read_firmware (camera->port);
-
+        firm = (char*)barbie_read_firmware (camera->port);
         sprintf(summary->text, _("Number of pictures: %i\nFirmware Version: %s"), num,firm);
-
         free(firm);
-
         return GP_OK;
 }
 
@@ -414,6 +398,5 @@ camera_init (Camera *camera, GPContext *context)
         res = barbie_ping (camera->port);
         if (res)
                 return (GP_OK);
-
         return (GP_ERROR);
 }
