@@ -14,13 +14,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
 
-#include <gphoto2-library.h>
+#include <gphoto2/gphoto2-library.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -152,9 +152,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		return (GP_ERROR_NOT_SUPPORTED);
 	}
 
-	CHECK_RESULT (gp_file_set_data_and_size (file, data, size));
-	CHECK_RESULT (gp_file_set_name (file, filename));
-
+	CHECK_RESULT (gp_file_set_data_and_size (file, (char*)data, size));
 	return (GP_OK);
 }
 
