@@ -81,6 +81,7 @@ digi_init (GPPort *port, CameraPrivateLibrary *priv)
 		if (catalog_tmp) priv->catalog = catalog_tmp;
 		else priv->catalog = catalog;
 	} else {
+		free (catalog);
 		priv->catalog = NULL;	/* We just have freed catalog_tmp */
 	}
 
@@ -197,12 +198,9 @@ int digi_delete_all     (GPPort *port, CameraPrivateLibrary *priv)
 {
 	int size;
 	int num_pics;
-	unsigned char *get_size;
+	unsigned char get_size[0x50];
 	unsigned char *junk=NULL;
 
-	get_size=malloc(0x50);
-	if(!get_size) 
-		return GP_ERROR_NO_MEMORY;
 	num_pics = priv->nb_entries;
 	GP_DEBUG("number of entries is %i\n", num_pics);
 	digi_reset (port);
@@ -221,7 +219,6 @@ int digi_delete_all     (GPPort *port, CameraPrivateLibrary *priv)
 						(get_size[0x43]<<24);
 	GP_DEBUG("size = 0x%x\n", size);
 	if(size <= 0xff) {
-		free(get_size);
 		GP_DEBUG("No size to read. This will not work.\n");
 		digi_reset(port);
 		return GP_OK;
@@ -237,4 +234,3 @@ int digi_delete_all     (GPPort *port, CameraPrivateLibrary *priv)
 
 	return GP_OK;
 }
-
