@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -31,9 +31,9 @@
 #ifdef OS2
 #include <db.h>
 #endif
-#include <netinet/in.h>
 
 #include "digita.h"
+#include "gphoto2-endian.h"
 
 #define GP_MODULE "digita"
 
@@ -162,8 +162,8 @@ int digita_get_file_data(CameraPrivateLibrary *dev, int thumbnail,
 
 	tbuf = malloc(GFD_BUFSIZE + sizeof(*gfdr));
 	if (!tbuf) {
-		GP_DEBUG("digita_get_file_data: unable to allocate %d bytes",
-			GFD_BUFSIZE + sizeof(*gfdr));
+		GP_DEBUG("digita_get_file_data: unable to allocate %ud bytes",
+			(unsigned int)(GFD_BUFSIZE + sizeof(*gfdr)));
 
 		return -1;
 	}
