@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -28,7 +28,7 @@
 #include <errno.h>
 #include <string.h>
 
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 #include "digita.h"
 
