@@ -36,8 +36,9 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  along with this program; if not, write to the
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA  02110-1301  USA
  */
 #include "config.h"
 #include "fuji.h"
@@ -46,7 +47,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #ifdef ENABLE_NLS
 #  include <libintl.h>
@@ -77,8 +78,8 @@
 
 #define GP_MODULE "fuji"
 
-#define CR(result)    {int r = (result); if (r < 0) return (r);}
-#define CRF(result,d) {int r = (result); if (r < 0) {free (d); return (r);}}
+#define CR(result)    {int __r = (result); if (__r < 0) return (__r);}
+#define CRF(result,d) {int __r = (result); if (__r < 0) {free (d); return (__r);}}
 #define CLEN(buf_len,required)					\
 {								\
 	if (buf_len < required) {				\
@@ -102,6 +103,8 @@
 #define FUJI_ACK 0x00
 #define FUJI_NAK 0x01
 
+static int fuji_reset (Camera *camera, GPContext *context);
+
 int
 fuji_ping (Camera *camera, GPContext *context)
 {
@@ -787,7 +790,7 @@ fuji_set_speed (Camera *camera, FujiSpeed speed, GPContext *context)
 	return (GP_OK);
 }
 
-int
+static int
 fuji_reset (Camera *camera, GPContext *context)
 {
 	unsigned char c = EOT;
