@@ -21,9 +21,12 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
+
+#define _BSD_SOURCE
+
 #include <config.h>
 
 #include <stdlib.h>
@@ -258,19 +261,17 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 	       CameraFile *file, void *user_data, GPContext *context)
 {
 	Camera *camera = user_data;
-	unsigned char *data = NULL;
-	int size, number, filetype;
+	int number, filetype;
 
 	CHECK (number = gp_filesystem_number (camera->fs, folder, filename, 
 				              context));
 	switch (type) {
 		case GP_FILE_TYPE_NORMAL:
-			CHECK (gsmart300_request_file (camera->pl, &data, 
-						       &size, number));
+			CHECK (gsmart300_request_file (camera->pl, file, number));
 			break;
 		case GP_FILE_TYPE_PREVIEW:
 			CHECK (gsmart300_request_thumbnail
-			       (camera->pl, &data, &size, number, &filetype));
+			       (camera->pl, file, number, &filetype));
 			if (filetype == GSMART_FILE_TYPE_IMAGE) {
 				CHECK (gp_file_set_mime_type (file, GP_MIME_BMP));
 			}
@@ -278,13 +279,6 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 		default:
 			return GP_ERROR_NOT_SUPPORTED;
 	}
-
-	if (!data)
-		return GP_ERROR;
-
-	CHECK (gp_file_set_data_and_size (file, data, size));
-	CHECK (gp_file_set_name (file, filename));
-
 	return GP_OK;
 }
 
@@ -303,11 +297,9 @@ get_info_func (CameraFilesystem *fs, const char *folder,
 
 	CHECK (gsmart300_get_file_info (camera->pl, n, &file));
 
-	info->file.fields = GP_FILE_INFO_NAME 
-		| GP_FILE_INFO_TYPE 
+	info->file.fields = GP_FILE_INFO_TYPE 
 		| GP_FILE_INFO_WIDTH 
 		| GP_FILE_INFO_HEIGHT;
-	strncpy (info->file.name, filename, sizeof (info->file.name));
 	if (file->mime_type == GSMART_FILE_TYPE_IMAGE) {
 		strcpy (info->file.type, GP_MIME_JPEG);
 		info->preview.width = 80;
