@@ -16,8 +16,9 @@
  *                                                                           *
  *  You should have received a copy of the GNU Library General Public        *
  *  License along with this program; see the file COPYING.LIB.  If not,      *
- *  write to the Free Software Foundation, Inc., 59 Temple Place -           *
- *  Suite 330,  Boston, MA 02111-1307, USA.                                  *
+ *  write to the 
+ *  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA  02110-1301  USA
  *                                                                           *
  *****************************************************************************/
 
@@ -52,6 +53,8 @@
      0x03 = Endcode   for a command
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -180,6 +183,8 @@ hp_gen_cmd_blob (enum hp215_cmd cmd, int bytes, unsigned char *argdata, unsigned
 	(*buf)[1] = cmd;
 	if (bytes >= 0x7d) {
 		gp_log (GP_LOG_ERROR, "hp215", "Using too large argument buffer %d bytes", bytes);
+		free (*buf);
+		*buf = NULL;
 		return GP_ERROR_BAD_PARAMETERS;
 	}
 	/* store arglen */
@@ -393,6 +398,7 @@ hp_get_timeDate_cam (Camera *cam, char *txtbuffer, size_t txtbuffersize)
 		return ret;
 	xmsg = msg;
 	if (msglen < 0x59) {
+		free (msg);
 		gp_log (GP_LOG_ERROR, "hp215", "too short reply block, %d bytes", msglen);
 		return GP_ERROR_IO;
 	}
@@ -437,7 +443,7 @@ hp_get_timeDate_cam (Camera *cam, char *txtbuffer, size_t txtbuffersize)
 	free (msg);
 
 
-	snprintf (txtbuffer, txtbuffersize, _("Current camera time:  %04d-%02d-%02d  %02d:%02d\nFree card memory: %d\nImages on card: %d\nImages free on card: %d\nBattery level: %d %%"),
+	snprintf (txtbuffer, txtbuffersize, _("Current camera time:  %04d-%02d-%02d  %02d:%02d\nFree card memory: %d\nImages on card: %d\nFree space (Images): %d\nBattery level: %d %%."),
 		date.year, date.month, date.day, date.hour, date.min,
 		freespace, usedimages, freeimages, percent
 	);
@@ -574,8 +580,11 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	free (buf);
 	if (ret < GP_OK)
 		return ret;
-	if (msglen < 2)
+
+	if (msglen < 2) {
+		free (msg);
 		return GP_ERROR_IO;
+	}
 
 /*
 0000  80 80 80 85 86 85-84 84 30 31 2f 30 32 2f  ..........01/02/
@@ -585,8 +594,10 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
  */
 	xmsg = msg;
 	ret = decode_u32(&xmsg, &msglen, &val);
-	if (ret < GP_OK)
+	if (ret < GP_OK) {
+		free (msg);
 		return ret;
+	}
 	memset (info, 0, sizeof(*info));
 	info->file.fields = GP_FILE_INFO_SIZE;
 	info->file.size = val;
@@ -599,8 +610,10 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	msglen	-= 2;
 
 	ret = decode_u32(&xmsg, &msglen, &val);
-	if (ret < GP_OK)
+	if (ret < GP_OK) {
+		free (msg);
 		return ret;
+	}
 	info->preview.fields = GP_FILE_INFO_SIZE;
 	info->preview.size = val;
 
