@@ -19,8 +19,8 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
 
 #ifndef __LIBRARY_H__
@@ -31,7 +31,7 @@
 #define SER_PKT_SIZE 4096
 #define USB_PKT_SIZE 4096
 
-#define CHECK(result) {int res; res = result; if (res < 0) return (res);}
+#define CHECK(result) {int __res; __res = result; if (__res < 0) return (__res);}
 
 struct jamcam_file {
 	int position;
@@ -41,12 +41,8 @@ struct jamcam_file {
 };
 
 int jamcam_enq(Camera *camera);
-int jamcam_query_mmc_card(Camera *camera);
-int jamcam_write_packet(Camera *camera, char *packet, int length);
-int jamcam_read_packet(Camera *camera, char *packet, int length);
 int jamcam_file_count(Camera *camera);
 struct jamcam_file *jamcam_file_info(Camera *camera, int number);
-int jamcam_fetch_memory(Camera *camera, CameraFile *file, char *data, int start, int length, GPContext *context );
 int jamcam_request_image(Camera *camera, CameraFile *file, char *buf, int *len, int number, GPContext *context );
 int jamcam_request_thumbnail(Camera *camera, CameraFile *file, char *buf, int *len, int number, GPContext *context );
 
