@@ -14,12 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef _JD11_DECOMP_H
 #define _JD11_DECOMP_H
-extern void picture_decomp_v1(char *compressed,char *uncompressed,int width,int height);
-extern void picture_decomp_v2(char *compressed,char *uncompressed,int width,int height);
+extern void picture_decomp_v1(unsigned char *compressed,unsigned char *uncompressed,int width,int height);
+extern void picture_decomp_v2(unsigned char *compressed,unsigned char *uncompressed,int width,int height);
 #endif
