@@ -1,3 +1,4 @@
+#define _POSIX_C_SOURCE 199309L
 #include "config.h"
 
 #include <stdio.h>
@@ -923,6 +924,7 @@ int dc210_open_card
 	return GP_OK;
 };
 
+#if 0 /* This function is not used by anybody */
 static int dc210_close_card
 (Camera * camera)
 {
@@ -935,6 +937,7 @@ static int dc210_close_card
 
 	return GP_OK;
 };
+#endif
 
 int dc210_download_picture_by_name
 (Camera * camera, CameraFile *file, const char *filename, dc210_picture_type type, GPContext *context)
@@ -959,8 +962,6 @@ int dc210_download_picture_by_name
 
 	dc210_cmd_packet_init(cmd_packet, filename);
 
-	gp_file_set_name(file, filename);
-
 	if (dc210_execute_command(camera, cmd) == GP_ERROR) return GP_ERROR;
 	if (dc210_write_command_packet(camera, cmd_packet) == GP_ERROR) return GP_ERROR;
 
