@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <stdio.h>
@@ -92,7 +94,7 @@ timeout_func (Camera *camera, GPContext *context)
 
 static int
 get_info (Camera *camera, unsigned int n, CameraFileInfo *info,
-	  CameraFile *file, GPContext *context)
+	  char *fn, CameraFile *file, GPContext *context)
 {
 	unsigned long image_id;
 	unsigned int buffer_size, exif_size;
@@ -117,20 +119,17 @@ get_info (Camera *camera, unsigned int n, CameraFileInfo *info,
 	strcpy (info->preview.type, GP_MIME_JPEG);
 
 	info->file.fields = GP_FILE_INFO_SIZE | GP_FILE_INFO_PERMISSIONS |
-			    GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME;
+			    GP_FILE_INFO_TYPE;
 	info->file.size = exif_size * 1000;
 	info->file.permissions = GP_FILE_PERM_READ;
 	if (!protected)
 		info->file.permissions |= GP_FILE_PERM_DELETE;
 	strcpy (info->file.type, GP_MIME_JPEG);
-	snprintf (info->file.name, sizeof (info->file.name),
-		  "%06i.jpeg", (int) image_id);
+	sprintf (fn, "%06i.jpeg", (int) image_id);
 
-	if (file) {
-		gp_file_set_type (file, GP_FILE_TYPE_EXIF);
-		gp_file_set_name (file, info->file.name);
-		gp_file_set_data_and_size (file, buffer, buffer_size);
-	} else
+	if (file)
+		gp_file_set_data_and_size (file, (char*)buffer, buffer_size);
+	else
 		free (buffer);
 
 	return (GP_OK);
@@ -157,10 +156,10 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	id = gp_context_progress_start (context, status.pictures,
 					_("Getting file list..."));
         for (i = 0; i < status.pictures; i++) {
-
+		char fn[40];
                 /* Get information */
 		gp_file_new (&file);
-		result = get_info (camera, i + 1, &info, file, context);
+		result = get_info (camera, i + 1, &info, fn, file, context);
 		if (result < 0) {
 			gp_file_unref (file);
 			return (result);
@@ -170,10 +169,9 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		 * Append directly to the filesystem instead of to the list,
 		 * because we have additional information.
 		 */
-		gp_filesystem_append (camera->fs, folder, info.file.name,
-				      context);
-		gp_filesystem_set_info_noop (camera->fs, folder, info, context);
-		gp_filesystem_set_file_noop (camera->fs, folder, file, context);
+		gp_filesystem_append (camera->fs, folder, fn, context);
+		gp_filesystem_set_info_noop (camera->fs, folder, fn, info, context);
+		gp_filesystem_set_file_noop (camera->fs, folder, fn, GP_FILE_TYPE_PREVIEW, file, context);
 		gp_file_unref (file);
 
 		gp_context_idle (context);
@@ -367,14 +365,6 @@ set_info_func (CameraFilesystem *fs, const char *folder, const char *file,
                 C(k_set_protect_status (camera->port, context,
 			camera->pl->image_id_long, image_id, protected));
         }
-
-	/* Name? */
-	if (info.file.fields & GP_FILE_INFO_NAME) {
-		gp_context_error (context, _("Your camera does not support "
-			"changing filenames."));
-		return (GP_ERROR_NOT_SUPPORTED);
-	}
-
         return (GP_OK);
 }
 
@@ -385,6 +375,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
         Camera *camera = data;
 	CameraFile *file;
         int n, result;
+	char fn[40];
 
 	/* We need image numbers starting with 1 */
 	n = gp_filesystem_number (camera->fs, folder, filename, context);
@@ -393,14 +384,13 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	n++;
 
 	gp_file_new (&file);
-	result = get_info (camera, n, info, file, context);
+	result = get_info (camera, n, info, fn, file, context);
 	if (result < 0) {
 		gp_file_unref (file);
 		return (result);
 	}
-	gp_filesystem_set_file_noop (fs, folder, file, context);
+	gp_filesystem_set_file_noop (fs, folder, filename, GP_FILE_TYPE_PREVIEW, file, context);
 	gp_file_unref (file);
-
         return (GP_OK);
 }
 
@@ -461,8 +451,9 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	image_id = atol (image_id_string);
 
 	/* Get information about the image */
-	C(gp_filesystem_get_info (camera->fs, folder,
-					filename, &info, context));
+	if (type == GP_FILE_TYPE_NORMAL)
+		C(gp_filesystem_get_info (camera->fs, folder,
+						filename, &info, context));
 
 	/*
 	 * Remove the timeout, get the image and start the timeout
@@ -474,25 +465,24 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		size = 2048;
 		r = k_get_image (camera->port, context,
 			camera->pl->image_id_long,
-			image_id, K_THUMBNAIL, (unsigned char **) &fdata,
+			image_id, K_THUMBNAIL, &fdata,
 			&size);
                 break;
         case GP_FILE_TYPE_NORMAL:
 		size = info.file.size;
 		r = k_get_image (camera->port, context,
 			camera->pl->image_id_long,
-			image_id, K_IMAGE_EXIF, (unsigned char **) &fdata,
+			image_id, K_IMAGE_EXIF, &fdata,
 			&size);
                 break;
         default:
-		r = GP_ERROR_NOT_SUPPORTED;
-		break;
+		return (GP_ERROR_NOT_SUPPORTED);
         }
 	C(r);
 	camera->pl->timeout = gp_camera_start_timeout (camera, PING_TIMEOUT,
 						       timeout_func);
 
-        C(gp_file_set_data_and_size (file, fdata, size));
+        C(gp_file_set_data_and_size (file, (char*)fdata, size));
         return gp_file_set_mime_type (file, GP_MIME_JPEG);
 }
 
@@ -552,7 +542,7 @@ camera_capture_preview (Camera* camera, CameraFile* file, GPContext *context)
         unsigned int size = 0;
 
         C(k_get_preview (camera->port, context, TRUE, &data, &size));
-        C(gp_file_set_data_and_size (file, data, size));
+        C(gp_file_set_data_and_size (file, (char*)data, size));
         C(gp_file_set_mime_type (file, GP_MIME_JPEG));
 
         return (GP_OK);
@@ -563,12 +553,13 @@ camera_capture (Camera* camera, CameraCaptureType type, CameraFilePath* path,
 		GPContext *context)
 {
         unsigned long image_id;
-	int exif_size;
+	unsigned int exif_size;
 	unsigned char *buffer = NULL;
 	unsigned int buffer_size;
 	int protected, r;
 	CameraFile *file = NULL;
 	CameraFileInfo info;
+	char fn[40];
 
 	C_NULL (camera && path);
 
@@ -594,22 +585,19 @@ camera_capture (Camera* camera, CameraCaptureType type, CameraFilePath* path,
 	strcpy (info.preview.type, GP_MIME_JPEG);
 
 	info.file.fields = GP_FILE_INFO_SIZE | GP_FILE_INFO_PERMISSIONS |
-			    GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME;
+			    GP_FILE_INFO_TYPE;
 	info.file.size = exif_size;
 	info.file.permissions = GP_FILE_PERM_READ;
 	if (!protected)
 		info.file.permissions |= GP_FILE_PERM_DELETE;
 	strcpy (info.file.type, GP_MIME_JPEG);
-	snprintf (info.file.name, sizeof (info.file.name),
-		  "%06i.jpeg", (int) image_id);
-	gp_filesystem_set_info_noop (camera->fs, path->folder, info, context);
+	sprintf (fn, "%06i.jpeg", (int) image_id);
+	gp_filesystem_set_info_noop (camera->fs, path->folder, fn, info, context);
 
 	gp_file_new (&file);
-	gp_file_set_name (file, info.file.name);
 	gp_file_set_mime_type (file, GP_MIME_JPEG);
-	gp_file_set_type (file, GP_FILE_TYPE_EXIF);
-	gp_file_set_data_and_size (file, buffer, buffer_size);
-	gp_filesystem_set_file_noop (camera->fs, path->folder, file, context);
+	gp_file_set_data_and_size (file, (char*)buffer, buffer_size);
+	gp_filesystem_set_file_noop (camera->fs, path->folder, fn, GP_FILE_TYPE_EXIF, file, context);
 	gp_file_unref (file);
 
         return (GP_OK);
