@@ -14,15 +14,15 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __KONICA_LOWLEVEL_H__
 #define __KONICA_LOWLEVEL_H__
 
-#include <gphoto2-port.h>
-#include <gphoto2-context.h>
+#include <gphoto2/gphoto2-port.h>
+#include <gphoto2/gphoto2-context.h>
 
 int l_ping (GPPort *, GPContext *);
 int l_init (GPPort *, GPContext *);
