@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  *
  * Modified by Aurlien Croc (APC) <programming@ap2c.com>
  * In particular : fix some bugs, and implementation of advanced 
@@ -33,6 +33,8 @@
  * information header and use this number with all functions !!
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 
 #include <string.h>
@@ -164,7 +166,7 @@ k_ping (GPPort *port) {
  */
 static int
 k_info_img (unsigned int image_no, void *data, CameraFileInfo* info, 
-		unsigned int *data_number)
+		int *data_number)
 {
 	unsigned char	cmd[6], buf[INFO_BUFFER];
 	Camera *camera = data;
@@ -177,9 +179,9 @@ k_info_img (unsigned int image_no, void *data, CameraFileInfo* info,
 	cmd[3] = 0x30 + ((image_no/100 )%10);
 	cmd[4] = 0x30 + ((image_no/10  )%10);
 	cmd[5] = 0x30 + ( image_no      %10);
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd)); 
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd)); 
 	if (ret<GP_OK) return ret;
-	ret = gp_port_read (camera->port, buf, INFO_BUFFER);
+	ret = gp_port_read (camera->port, (char*)buf, INFO_BUFFER);
 	if (ret<GP_OK) return ret;
 
 	/* Search the image data number into the memory */
@@ -202,7 +204,7 @@ k_info_img (unsigned int image_no, void *data, CameraFileInfo* info,
 
 	/* Image information */
 	info->file.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_SIZE
-		| GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT | GP_FILE_INFO_NAME
+		| GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT
 		| GP_FILE_INFO_PERMISSIONS;
 	strcpy (info->file.type, GP_MIME_JPEG);
 	info->file.size = ((buf[IMAGE_SIZE_PTR] << 24) |
@@ -210,8 +212,6 @@ k_info_img (unsigned int image_no, void *data, CameraFileInfo* info,
 		| buf[IMAGE_SIZE_PTR+3]);
 	info->file.width = IMAGE_WIDTH;
 	info->file.height = IMAGE_HEIGHT;
-	snprintf (info->file.name, sizeof (info->file.name),(char *) FILENAME,
-		image_no);
 	if (buf[IMAGE_PROTECTION_FLAG] == IMAGE_PROTECTED)
 		info->file.permissions = GP_FILE_PERM_READ;
 	else
@@ -242,9 +242,9 @@ k_getdata (int image_no, int type, unsigned int len, void *data,
 	cmd[5] = 0x30 + ((image_no/10  )%10);
 	cmd[6] = 0x30 + ( image_no      %10);
 
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK) return ret;
-	ret = gp_port_read (camera->port, &ack, ACK_LEN);
+	ret = gp_port_read (camera->port, (char*)&ack, ACK_LEN);
 	if (ret < GP_OK) return ret;
 	if (ack == NACK) {
 		gp_context_error(context, _("This preview doesn't exist."));
@@ -259,13 +259,13 @@ k_getdata (int image_no, int type, unsigned int len, void *data,
 		unsigned char csum;
 		int xret;
 
-		xret = gp_port_read (camera->port, buf, DATA_BUFFER);
+		xret = gp_port_read (camera->port, (char*)buf, DATA_BUFFER);
 		if (xret < GP_OK) {
 			if (type == GP_FILE_TYPE_NORMAL)
 				gp_context_progress_stop (context, id);
 			return xret; 
 		}
-		ret = gp_port_read (camera->port, &csum, CSUM_LEN);
+		ret = gp_port_read (camera->port, (char*)&csum, CSUM_LEN);
 		if (ret < GP_OK) {
 			if (type == GP_FILE_TYPE_NORMAL)
 				gp_context_progress_stop (context, id);
@@ -276,7 +276,7 @@ k_getdata (int image_no, int type, unsigned int len, void *data,
 				gp_context_progress_stop (context, id);
 			/* acknowledge the packet */
 			ack = NACK;
-			ret = gp_port_write (camera->port, &ack, ACK_LEN);
+			ret = gp_port_write (camera->port, (char*)&ack, ACK_LEN);
 			if (ret < GP_OK)
 				return ret;
 			gp_context_error(context, _("Data has been corrupted."));
@@ -292,13 +292,13 @@ k_getdata (int image_no, int type, unsigned int len, void *data,
 
 		/* acknowledge the packet */
 		ack = ACK; 
-		ret = gp_port_write (camera->port, &ack, ACK_LEN);
+		ret = gp_port_write (camera->port, (char*)&ack, ACK_LEN);
 		if (ret < GP_OK) {
 			if (type == GP_FILE_TYPE_NORMAL)
 				gp_context_progress_stop (context, id);
 			return ret;
 		}
-		ret = gp_port_read (camera->port, &state, STATE_LEN);
+		ret = gp_port_read (camera->port, (char*)&state, STATE_LEN);
 		if (ret < GP_OK) {
 			if (type == GP_FILE_TYPE_NORMAL)
 				gp_context_progress_stop (context, id);
@@ -312,7 +312,7 @@ k_getdata (int image_no, int type, unsigned int len, void *data,
 	}
 	/* acknowledge the packet */
 	ack = ACK; 
-	ret = gp_port_write (camera->port, &ack, ACK_LEN);
+	ret = gp_port_write (camera->port, (char*)&ack, ACK_LEN);
 	if (ret < GP_OK) {
 		if (type == GP_FILE_TYPE_NORMAL)
 			gp_context_progress_stop (context, id);
@@ -335,7 +335,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	       GPContext *context)
 {
 	unsigned char *d,*thumbnail;
-	int image_no, len, ret, image_number;
+	int image_number, image_no, len, ret;
 	long long_len;
 	CameraFileInfo file_info;
 	exifparser exifdat;
@@ -398,11 +398,8 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 				_("Image type %d is not supported by this camera !"), type);
 			return (GP_ERROR_NOT_SUPPORTED);
 	}
-	gp_file_set_name (file, filename);
 	gp_file_set_mime_type (file, GP_MIME_JPEG);
-	if (type == GP_FILE_TYPE_EXIF)
-		gp_file_set_type (file, GP_FILE_TYPE_EXIF);
-	ret = gp_file_append(file, d, len);
+	ret = gp_file_append(file, (char*)d, len);
 	free(d);
 	return (ret);
 }
@@ -418,7 +415,8 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	Camera *camera = data;
 	CameraFileInfo file_info;
 	unsigned char cmd[7], ack;
-	int image_no, ret;
+	int image_no;
+	int ret;
 
 	GP_DEBUG ("*** ENTER: delete_file_func ***");
 
@@ -446,9 +444,9 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	cmd[4] = 0x30 + ((image_no/100 )%10);
 	cmd[5] = 0x30 + ((image_no/10  )%10);
 	cmd[6] = 0x30 + ( image_no      %10);
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK) return ret;
-	ret = gp_port_read (camera->port, &ack, ACK_LEN);
+	ret = gp_port_read (camera->port, (char*)&ack, ACK_LEN);
 	if (ret<GP_OK) return ret;
 	if (ack != ACK) {
 		gp_context_error(context, _("Can't delete image %s."),filename);
@@ -477,9 +475,9 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
 	cmd[4] = 0x30;
 	cmd[5] = 0x30;
 	cmd[6] = 0x30;
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK) return ret;
-	ret = gp_port_read (camera->port, &ack, ACK_LEN);
+	ret = gp_port_read (camera->port, (char*)&ack, ACK_LEN);
 	if (ret<GP_OK) return ret;
 	if (ack != ACK) {
 		gp_context_error(context, _("Can't delete all images."));
@@ -495,7 +493,8 @@ delete_all_func (CameraFilesystem *fs, const char *folder, void *data,
  * Upload an image to the camera
  */
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, 
+put_file_func (CameraFilesystem *fs, const char *folder, const char *name,
+		CameraFileType type, CameraFile *file, 
 		void *data, GPContext *context)
 {
 	Camera *camera = data;
@@ -510,13 +509,13 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 	/* Send function */
 	cmd[0] = ESC;
 	cmd[1] = UPLOADDATA;
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd)); 
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd)); 
 	if (ret<GP_OK) 
 		return ret;
 	gp_file_get_data_and_size(file, &d, &len);
 	id = gp_context_progress_start (context, len, _("Uploading image..."));
 	for (i=0; i < ((len+DATA_BUFFER-1) / DATA_BUFFER); i++) {
-		ret = gp_port_read (camera->port, &ack, ACK_LEN);
+		ret = gp_port_read (camera->port, (char*)&ack, ACK_LEN);
 		if (ret<GP_OK) {
 			gp_context_progress_stop (context, id);
 			return ret;
@@ -529,14 +528,14 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 			return (GP_ERROR);
 		}
 		state = NEXTFRAME;
-		ret = gp_port_write (camera->port, &state, STATE_LEN); 
+		ret = gp_port_write (camera->port, (char*)&state, STATE_LEN); 
 		if (ret<GP_OK) {
 			gp_context_progress_stop (context, id);
 			return ret;
 		}
 		if ((len - len_sent) <= DATA_BUFFER) {
 			/* Send the last datas */
-			ret = gp_port_write (camera->port, &d[i*DATA_BUFFER],
+			ret = gp_port_write (camera->port, (char*)&d[i*DATA_BUFFER],
 				(len - len_sent));
 			if (ret<GP_OK) {
 				gp_context_progress_stop (context, id);
@@ -544,7 +543,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 			}
 			/* and complete with zeros */
 			bzero(buf,DATA_BUFFER);
-			ret = gp_port_write (camera->port, buf, (DATA_BUFFER - 
+			ret = gp_port_write (camera->port, (char*)buf, (DATA_BUFFER - 
 				(len - len_sent)));
 			if (ret<GP_OK) {
 				gp_context_progress_stop (context, id);
@@ -569,7 +568,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 				(unsigned char *)&d[i*DATA_BUFFER],
 				DATA_BUFFER);
 		}
-		ret = gp_port_write (camera->port, &sum, CSUM_LEN); 
+		ret = gp_port_write (camera->port, (char*)&sum, CSUM_LEN); 
 		if (ret<GP_OK) {
 			gp_context_progress_stop (context, id);
 			return ret;
@@ -577,12 +576,12 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 		gp_context_progress_update (context, id, len_sent);
 	}
 	state = EOT;
-	ret = gp_port_write (camera->port, &state, STATE_LEN); 
+	ret = gp_port_write (camera->port, (char*)&state, STATE_LEN); 
 	if (ret<GP_OK) {
 		gp_context_progress_stop (context, id);
 		return ret;
 	}
-	ret = gp_port_read (camera->port, &ack, ACK_LEN);
+	ret = gp_port_read (camera->port, (char*)&ack, ACK_LEN);
 	if (ret<GP_OK) {
 		gp_context_progress_stop (context, id);
 		return ret;
@@ -612,10 +611,10 @@ camera_capture (Camera* camera, CameraCaptureType type, CameraFilePath* path,
 	/* Just check if there is space available yet */
 	cmd[0] = ESC;
 	cmd[1] = GETCAMINFO;
-	ret = gp_port_write (camera->port, cmd, 2); 
+	ret = gp_port_write (camera->port, (char*)cmd, 2); 
 	if (ret<GP_OK)
 		return ret;
-	ret = gp_port_read (camera->port, buf, INFO_BUFFER);
+	ret = gp_port_read (camera->port, (char*)buf, INFO_BUFFER);
 	nbr_images = (buf[FREE_IMAGE_PTR] << 8) | (buf[FREE_IMAGE_PTR+1]);
 	images_taken = (buf[TAKEN_IMAGE_PTR] << 8) | (buf[TAKEN_IMAGE_PTR+1]);
 
@@ -623,22 +622,22 @@ camera_capture (Camera* camera, CameraCaptureType type, CameraFilePath* path,
 	cmd[0] = ESC;
 	cmd[1] = CAPTUREIMAGE_CMD1;
 	cmd[2] = CAPTUREIMAGE_CMD2;
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK)
 		return ret;
-	ret = gp_port_read (camera->port, &ack, ACK_LEN);
+	ret = gp_port_read (camera->port, (char*)&ack, ACK_LEN);
 	if (ret<GP_OK)
 		return ret;
 	if (ack == NACK) {
 		if (buf[CAMERA_MODE_PTR] != REC_MODE)
 			gp_context_error(context, _("You must be in record "
-				"mode to capture image."));
+				"mode to capture images."));
 		else if (!nbr_images)
 			gp_context_error(context, _("No space available "
-				"to capture new image. You must delete some "
+				"to capture new images. You must delete some "
 				"images."));
 		else
-			gp_context_error(context, _("Can't capture new image. "
+			gp_context_error(context, _("Can't capture new images. "
 				"Unknown error"));
 		return (GP_ERROR);
 	}
@@ -697,10 +696,10 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 
 	cmd[0] = ESC;
 	cmd[1] = GETCAMINFO;
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK)
 		return ret;
-	ret = gp_port_read (camera->port, buf, INFO_BUFFER);
+	ret = gp_port_read (camera->port, (char*)buf, INFO_BUFFER);
 	if (ret<GP_OK)
 		return ret;
 	num = (buf[TAKEN_IMAGE_PTR] << 8) | (buf[TAKEN_IMAGE_PTR+1]);
@@ -728,10 +727,10 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
 	/* get informations about camera */
 	cmd[0] = ESC;
 	cmd[1] = GETCAMINFO;
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK)
 		return ret;
-	ret = gp_port_read (camera->port, buf, INFO_BUFFER);
+	ret = gp_port_read (camera->port, (char*)buf, INFO_BUFFER);
 	if (ret<GP_OK)
 		return ret;
 
@@ -934,22 +933,22 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
        /* Focus */
         gp_widget_new (GP_WIDGET_RADIO, _("Focus"), &widget);
         gp_widget_append (section, widget);
-        gp_widget_add_choice (widget, _("2.0m"));
-	gp_widget_add_choice (widget, _("0.5m"));
-	gp_widget_add_choice (widget, _("0.1m"));
+        gp_widget_add_choice (widget, _("2.0 m"));
+	gp_widget_add_choice (widget, _("0.5 m"));
+	gp_widget_add_choice (widget, _("0.1 m"));
         gp_widget_add_choice (widget, _("Auto"));
         switch (buf[FOCUS_PTR]) {
 		case 0:
 			gp_widget_set_value (widget, _("Auto"));
 			break;
 		case 1:
-	                gp_widget_set_value (widget, _("2.0m"));
+	                gp_widget_set_value (widget, _("2.0 m"));
 	                break;
 	        case 2:
-        	        gp_widget_set_value (widget, _("0.5m"));
+        	        gp_widget_set_value (widget, _("0.5 m"));
                 	break;
 	        case 3:
-        	        gp_widget_set_value (widget, _("0.1m"));
+        	        gp_widget_set_value (widget, _("0.1 m"));
                 	break;
         }
 
@@ -957,14 +956,14 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         gp_widget_new (GP_WIDGET_RADIO, _("White balance"), &widget);
         gp_widget_append (section, widget);
         gp_widget_add_choice (widget, _("Office"));
-	gp_widget_add_choice (widget, _("Day-lt"));
+	gp_widget_add_choice (widget, _("Daylight"));
 	gp_widget_add_choice (widget, _("Auto"));
         switch (buf[WHITE_BALANCE_PTR]) {
 	        case 0:
         	        gp_widget_set_value (widget, _("Auto"));
                 	break;
 	        case 1:
-        	        gp_widget_set_value (widget, _("Day-lt"));
+        	        gp_widget_set_value (widget, _("Daylight"));
                 	break;
 	        case 2:
         	        gp_widget_set_value (widget, _("Office"));
@@ -1088,7 +1087,7 @@ camera_get_config (Camera* camera, CameraWidget** window, GPContext *context)
         /* Self Timer */
         gp_widget_new (GP_WIDGET_RADIO, _("Self Timer"), &widget);
         gp_widget_append (section, widget);
-        gp_widget_add_choice (widget, _("Self Timer (only next picture)"));
+        gp_widget_add_choice (widget, _("Self Timer (next picture only)"));
         gp_widget_add_choice (widget, _("Normal"));
         switch (buf[TIMER_PTR]) {
 	        case 1:
@@ -1113,7 +1112,8 @@ static int
 camera_summary (Camera *camera, CameraText *text, GPContext *context)
 {
 	unsigned char cmd[2], buf[INFO_BUFFER];
-	unsigned char power[20],mode[20],date_disp[20],date[50];
+	char date_disp[20],date[50];
+	char power[20],mode[20];
 	int ret,capacity=0,autopoweroff=0,image_taken=0,image_remained=0;
 	time_t timestamp=0;
 	struct tm tmp;
@@ -1123,10 +1123,10 @@ camera_summary (Camera *camera, CameraText *text, GPContext *context)
 	/* get informations about camera */
 	cmd[0] = ESC;
 	cmd[1] = GETCAMINFO;
-	ret = gp_port_write (camera->port, cmd, sizeof(cmd));
+	ret = gp_port_write (camera->port, (char*)cmd, sizeof(cmd));
 	if (ret<GP_OK)
 		return ret;
-	ret = gp_port_read (camera->port, buf, INFO_BUFFER);
+	ret = gp_port_read (camera->port, (char*)buf, INFO_BUFFER);
 	if (ret<GP_OK)
 		return ret;
 
@@ -1162,9 +1162,9 @@ camera_summary (Camera *camera, CameraText *text, GPContext *context)
 	}
 	snprintf(text->text, sizeof(text->text), 
 			_("Model: %s\n"
-			"Capacity: %iMo\n"
+			"Capacity: %i Mb\n"
 			"Power: %s\n"
-			"Auto Off Time: %imin\n"
+			"Auto Off Time: %i min\n"
 			"Mode: %s\n"
 			"Images: %i/%i\n"
 			"Date display: %s\n"
@@ -1199,14 +1199,13 @@ static int
 camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
 	snprintf(manual->text,sizeof(manual->text),
-		_("About Konica Q-M150:\n"
-		"This camera does not allow any changes\n"
-		"from the outside. So in the configuration, you can\n"
-		"just see what it is configured on the camera\n"
-		"but you can't change anything.\n\n"
-		"If you have some issues with this driver, please\n"
-		"send an e-mail to the authors of this driver.\n\n"
-		"Thanks to use GPhoto2 and UNIX/Linux Operating systems.\n"));
+	_("About Konica Q-M150:\n"
+	"This camera does not allow any changes\n"
+	"from the outside. So in the configuration, you can\n"
+	"only see what it is configured on the camera\n"
+	"but you can not change anything.\n\n"
+	"If you have some issues with this driver, please e-mail its authors.\n"
+	));
 	return (GP_OK);
 }
 
