@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -94,7 +94,8 @@ void largan_pict_free (largan_pict_info * ptr)
 }
 
 
-void largan_pict_alloc_data (largan_pict_info * ptr, uint32_t size)
+static void
+largan_pict_alloc_data (largan_pict_info * ptr, uint32_t size)
 {
 	ptr->data = realloc (ptr->data, size);
 	ptr->data_size = size;
@@ -174,9 +175,6 @@ int largan_get_pict (Camera * camera, largan_pict_type type,
 		return ret;
 	}
         ret = largan_recv_reply (camera, &reply, &code, NULL);
-	if (ret < 0) {
-		return ret;
-	}
 /* here we have to receive 7 bytes back 
  the 2nd contains 00 for thumbnail or 01 for original picture
  the 3rd contains the picturenumber for an original picture
@@ -424,7 +422,7 @@ static int largan_recv_reply (Camera * camera, uint8_t *reply,
 		break;
 	default:
 		packet_size = 0;
-		GP_DEBUG("largan_receive_reply: Unkown reply.\n");
+		GP_DEBUG("largan_receive_reply: Unknown reply.\n");
 		break;
 	}
 	if (reply) {
