@@ -16,13 +16,17 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include <config.h>
 #include <stdio.h>
+#include <unistd.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
@@ -63,7 +67,7 @@ mars_init (Camera *camera, GPPort *port, Info *info)
 {
 	char c[16];
 	unsigned char status = 0;
-	memset(info,0, sizeof(info)); 
+	memset(info,0, sizeof(*info)); 
 	memset(c,0,sizeof(c));
 	GP_DEBUG("Running mars_init\n");
 
