@@ -1,5 +1,5 @@
 /*
-	$Id: dc.h 7736 2005-04-05 21:04:19Z hun $
+	$Id: dc.h 9336 2006-10-15 00:46:42Z hfiguiere $
 
 	Copyright  2000 Mariusz Zynel <mariusz@mizar.org> (gPhoto port)
 	Copyright  2000 Fredrik Roubert <roubert@df.lth.se> (idea)
@@ -23,7 +23,7 @@
 #include <stdio.h>
 #include <errno.h>
 #include <_stdint.h>
-#include <gphoto2.h>
+#include <gphoto2/gphoto2.h>
 
 #define DSC1	1	/* DC1000 */
 #define DSC2	2	/* DC1580 */
@@ -40,8 +40,8 @@ struct _CameraPrivateLibrary {
 	int			size;
 };
 
-#define DSC_BLOCKSIZE	  0x400   /* amount of image data transfered in a single packet */
-#define DSC_BUFSIZE	  0x406   /* largest possible amount of data transfered in a single packet */
+#define DSC_BLOCKSIZE	  0x400   /* amount of image data transferred in a single packet */
+#define DSC_BUFSIZE	  0x406   /* largest possible amount of data transferred in a single packet */
 #define DSC_MAXIMAGESIZE  0xfffff /* largest possible file that can be uploaded */
 #define DSC_FILENAMEFMT	  "dsc%04i.jpg" /* format of image file names */
 #define DSC_THUMBNAMEFMT  "dsc%04i-thumbnail.jpg" /* format of thumbnail file names */
