@@ -19,8 +19,8 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
 
 #include <config.h>
@@ -99,13 +99,6 @@ camera_abilities (CameraAbilitiesList *list)
 	return GP_OK;
 }
 
-static int
-camera_exit (Camera *camera, GPContext *context)
-{
-	/* Nothing to do, really, simply get out of here. */
-	return GP_OK;
-}
-
 static int
 file_list_func (CameraFilesystem *fs, const char *folder,
 		CameraList *list, void *data, GPContext *context)
@@ -115,7 +108,8 @@ file_list_func (CameraFilesystem *fs, const char *folder,
 	CameraFile *file;
 	CameraFileInfo info;
 	unsigned char *buffer = NULL;
-	int n_img=0, n_avi=0, n_wav=0;
+	int ret, n_img=0, n_avi=0, n_wav=0;
+	char fn[100];
 
 	CHECK (pccam300_get_filecount (camera->port, &filecount));
 
@@ -126,64 +120,49 @@ file_list_func (CameraFilesystem *fs, const char *folder,
 		/* Get information */
 		gp_file_new (&file);
 	
-		CHECK (pccam300_get_file (camera->port, context, i,
-		                          &buffer, &size, &type));
+		ret = pccam300_get_file (camera->port, context, i,
+		                          &buffer, &size, &type);
+		if (ret < GP_OK) {
+			gp_file_free (file);
+			return ret;
+		}
 
 		info.audio.fields = GP_FILE_INFO_NONE;
 		info.preview.fields = GP_FILE_INFO_NONE;
 
-		info.file.fields = GP_FILE_INFO_SIZE |
-			GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME;
+		info.file.fields = GP_FILE_INFO_SIZE | GP_FILE_INFO_TYPE;
 		info.file.size = size;
 
 		switch (type) {
 			case PCCAM300_MIME_JPEG:
 				strcpy (info.file.type, GP_MIME_JPEG);
-				snprintf (info.file.name, 
-				          sizeof (info.file.name),
-				          "Image%03i.jpeg", n_img++);
+				sprintf (fn, "Image%03i.jpeg", n_img++);
 				break;
 			case PCCAM300_MIME_AVI:
 				strcpy (info.file.type, GP_MIME_AVI);
-				snprintf (info.file.name, 
-				          sizeof (info.file.name),
-				          "Movie%03i.UNUSABLE", n_avi++);
+				sprintf (fn, "Movie%03i.UNUSABLE", n_avi++);
 				break;
 			case PCCAM300_MIME_WAV:
 				strcpy (info.file.type, GP_MIME_WAV);
-				snprintf (info.file.name, 
-				          sizeof (info.file.name),
-				          "Audio%03i.UNUSABLE", n_wav++);
+				sprintf (fn, "Audio%03i.UNUSABLE", n_wav++);
 				break;
 			default:
 				break;
 		}
 
-		if (file) {
-			gp_file_set_type (file, GP_FILE_TYPE_NORMAL);
-			gp_file_set_name (file, info.file.name);
+		if (file)
 			gp_file_set_data_and_size (file, buffer, size);
-		} else
+		else
 			free (buffer);
 		
 		/*
 		 * Append directly to the filesystem instead of to the list,
 		 * because we have additional information. 
 		 * */
-		gp_filesystem_append (camera->fs, folder, info.file.name,
-		                      context);
-		gp_filesystem_set_info_noop (camera->fs, folder, info,
-		                             context);
-		/* FIXME: This is disabled for now, due to it seeming to
-		 *        cause corruption within libgphoto itself.
-		 *        A side effect of this is that file caching does
-		 *        not happen, so files must be downloaded twice if
-		 *        they are to be saved to disk.
-		 *        NWG: Sun 19th January 2003.
-		 *
-		 * gp_filesystem_set_file_noop (camera->fs, folder, file,
-		 *                              context);
-		 */
+		gp_filesystem_append (camera->fs, folder, fn, context);
+		gp_filesystem_set_info_noop (camera->fs, folder, fn, info, context);
+		gp_filesystem_set_file_noop (camera->fs, folder, fn, GP_FILE_TYPE_NORMAL,
+					file, context);
 		gp_file_unref (file);
 
 		gp_context_idle (context);
@@ -202,7 +181,8 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 {
 	Camera *camera = user_data;
 	unsigned char *data = NULL;
-	unsigned int size, index, mimetype;
+	unsigned int size, mimetype;
+	int index;
 
 	size = 0;
 	index = gp_filesystem_number (fs, folder, filename, context);
@@ -217,9 +197,7 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 		default:
 			return GP_ERROR_NOT_SUPPORTED;
 	}
-	gp_file_set_data_and_size (file, data, size);
-	gp_file_set_name (file, filename);
-	return GP_OK;
+	return gp_file_set_data_and_size (file, data, size);
 }
 
 static int
@@ -242,13 +220,6 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 	return GP_OK;
 }
 
-static int
-camera_manual (Camera *camera, CameraText *manual, GPContext *context)
-{
-	strcpy (manual->text, _("Manual Not Implemented Yet"));
-	return GP_OK;
-}
-
 static int
 camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
@@ -305,9 +276,7 @@ camera_init (Camera *camera, GPContext *context)
 	GPPortSettings settings;
 	int ret = 0;
 
-	camera->functions->exit = camera_exit;
 	camera->functions->summary = camera_summary;
-	camera->functions->manual = camera_manual;
 	camera->functions->about = camera_about;
 	gp_log (GP_LOG_DEBUG, "pccam 300", "Initializing the camera\n");
 	switch (camera->port->type) {
@@ -318,7 +287,7 @@ camera_init (Camera *camera, GPContext *context)
 			settings.usb.inep = 0x82;
 			settings.usb.outep = 0x03;
 			settings.usb.config = 1;
-			settings.usb.interface = 1;
+			settings.usb.interface = 0;
 			settings.usb.altsetting = 0;
 			ret = gp_port_set_settings (camera->port, settings);
 			if (ret < 0)
