@@ -1,7 +1,7 @@
 /* usb.c
  *
  * Copyright (C) 2001-2004 Mariusz Woloszyn <emsi@ipartners.pl>
- * Copyright (C) 2003-2007 Marcus Meissner <marcus@jet.franken.de>
+ * Copyright (C) 2003-2011 Marcus Meissner <marcus@jet.franken.de>
  * Copyright (C) 2006-2007 Linus Walleij <triad@df.lth.se>
  *
  * This library is free software; you can redistribute it and/or
@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #define _BSD_SOURCE
@@ -61,7 +61,7 @@
 
 /* PTP2_FAST_TIMEOUT: how long (in milliseconds) we should wait for
  * an URB to come back on an interrupt endpoint */
-#define PTP2_FAST_TIMEOUT       50
+#define PTP2_FAST_TIMEOUT       100
 
 /* Pack / unpack functions */
 
@@ -102,9 +102,9 @@ ptp_usb_sendreq (PTPParams* params, PTPContainer* req)
 
 uint16_t
 ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
-		  unsigned long size, PTPDataHandler *handler
+		  uint64_t size, PTPDataHandler *handler
 ) {
-	uint16_t ret;
+	uint16_t ret = PTP_RC_OK;
 	int res, wlen, datawlen;
 	PTPUSBBulkContainer usbdata;
 	unsigned long bytes_left_to_transfer, written;
@@ -128,7 +128,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 		/* For all camera devices. */
 		datawlen = (size<PTP_USB_BULK_PAYLOAD_LEN_WRITE)?size:PTP_USB_BULK_PAYLOAD_LEN_WRITE;
 		wlen = PTP_USB_BULK_HDR_LEN + datawlen;
-		ret = handler->getfunc(params, handler->private, datawlen, usbdata.payload.data, &gotlen);
+		ret = handler->getfunc(params, handler->priv, datawlen, usbdata.payload.data, &gotlen);
 		if (ret != PTP_RC_OK)
 			return ret;
 		if (gotlen != datawlen)
@@ -161,7 +161,7 @@ ptp_usb_senddata (PTPParams* params, PTPContainer* ptp,
 		toread = 4096;
 		if (toread > bytes_left_to_transfer)
 			toread = bytes_left_to_transfer;
-		ret = handler->getfunc (params, handler->private, toread, bytes, &readlen);
+		ret = handler->getfunc (params, handler->priv, toread, bytes, &readlen);
 		if (ret != PTP_RC_OK)
 			break;
 		res = gp_port_write (camera->port, (char*)bytes, readlen);
@@ -227,7 +227,7 @@ retry:
 		gp_log (GP_LOG_DEBUG, "ptp2/usbread", "Clearing halt on IN EP and retrying once.");
 		gp_port_usb_clear_halt (camera->port, GP_PORT_USB_ENDPOINT_IN);
 		/* retrying only makes sense if we did not read anything yet */
-		if ((tries++ < 1) && (result == 0))
+		if (tries++ < 1)
 			goto retry;
 	}
 	return PTP_ERROR_IO;
@@ -273,7 +273,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		if (dtoh16(usbdata.code)!=ptp->Code) {
 			/* A creative Zen device breaks down here, by leaving out
 			 * Code and Transaction ID */
-			if (MTP_ZEN_BROKEN_HEADER(camera->pl)) {
+			if (MTP_ZEN_BROKEN_HEADER(params)) {
 				gp_log (GP_LOG_DEBUG, "ptp2/ptp_usb_getdata", "Read broken PTP header (Code is %04x vs %04x), compensating.",
 					dtoh16(usbdata.code), ptp->Code
 				);
@@ -290,6 +290,11 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		if (usbdata.length == 0xffffffffU) {
 			unsigned char	*data = malloc (PTP_USB_BULK_HS_MAX_PACKET_LEN_READ);
 			if (!data) return PTP_RC_GeneralError;
+			/* Copy first part of data to 'data' */
+			handler->putfunc(
+				params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
+				&written
+			);
 			/* stuff data directly to passed data handler */
 			while (1) {
 				unsigned long written;
@@ -298,7 +303,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 					free (data);
 					return PTP_ERROR_IO;
 				}
-				handler->putfunc (params, handler->private, result, data, &written);
+				handler->putfunc (params, handler->priv, result, data, &written);
 				if (result < PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) 
 					break;
 			}
@@ -346,7 +351,7 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 
 		/* Copy first part of data to 'data' */
 		handler->putfunc(
-			params, handler->private, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
+			params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
 			&written
 		);
 		/* Is that all of data? */
@@ -365,7 +370,6 @@ retry:
 		curread = 0; res = 0;
 		while (bytes_to_read > 0) {
 			unsigned long toread = bytes_to_read;
-			int res;
 
 			/* read in large blobs.
 			 * if smaller than large blob, read all but the last short packet
@@ -380,7 +384,7 @@ retry:
 				ret = PTP_ERROR_IO;
 				break;
 			}
-			ret = handler->putfunc (params, handler->private,
+			ret = handler->putfunc (params, handler->priv,
 				res, data, &written
 			);
 			if (ret != PTP_RC_OK)
@@ -429,7 +433,6 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 {
 	uint16_t 		ret;
 	unsigned long		rlen;
-	Camera			*camera = ((PTPData *)params->data)->camera;
 	PTPUSBBulkContainer	usbresp;
 	/*GPContext		*context = ((PTPData *)params->data)->context;*/
 
@@ -448,7 +451,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 		ret = dtoh16(usbresp.code);
 	}
 	if (ret!=PTP_RC_OK) {
-		gp_log (GP_LOG_ERROR, "ptp2/usb_getresp","request code 0x%04x getting resp error 0x%04x", resp->Code, ret);
+		gp_log (GP_LOG_DEBUG, "ptp2/usb_getresp","request code 0x%04x getting resp error 0x%04x", resp->Code, ret);
 		return ret;
 	}
 	/* build an appropriate PTPContainer */
@@ -456,7 +459,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 	resp->SessionID=params->session_id;
 	resp->Transaction_ID=dtoh32(usbresp.trans_id);
 	if (resp->Transaction_ID != params->transaction_id - 1) {
-		if (MTP_ZEN_BROKEN_HEADER(camera->pl)) {
+		if (MTP_ZEN_BROKEN_HEADER(params)) {
 			gp_log (GP_LOG_DEBUG, "ptp2/ptp_usb_getresp", "Read broken PTP header (transid is %08x vs %08x), compensating.",
 				resp->Transaction_ID, params->transaction_id - 1
 			);
@@ -464,7 +467,7 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 		}
 		/* else will be handled by ptp.c as error. */
 	}
-
+	resp->Nparam=(rlen-12)/4;
 	resp->Param1=dtoh32(usbresp.payload.params.param1);
 	resp->Param2=dtoh32(usbresp.payload.params.param2);
 	resp->Param3=dtoh32(usbresp.payload.params.param3);
@@ -482,11 +485,16 @@ ptp_usb_getresp (PTPParams* params, PTPContainer* resp)
 static inline uint16_t
 ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 {
-	int			result, timeout;
+	int			result, timeout, fasttimeout;
 	unsigned long		rlen;
 	PTPUSBEventContainer	usbevent;
 	Camera			*camera = ((PTPData *)params->data)->camera;
 
+	if (params->deviceinfo.VendorExtensionID == PTP_VENDOR_CANON)
+		fasttimeout = PTP2_FAST_TIMEOUT*2;
+	else
+		fasttimeout = PTP2_FAST_TIMEOUT;
+
 	PTP_CNT_INIT(usbevent);
 
 	if (event==NULL)
@@ -499,8 +507,9 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 		break;
 	case PTP_EVENT_CHECK_FAST:
 		gp_port_get_timeout (camera->port, &timeout);
-		gp_port_set_timeout (camera->port, PTP2_FAST_TIMEOUT);
+		gp_port_set_timeout (camera->port, fasttimeout);
 		result = gp_port_check_int (camera->port, (char*)&usbevent, sizeof(usbevent));
+		if (result <= 0) result = gp_port_check_int (camera->port, (char*)&usbevent, sizeof(usbevent));
 		gp_port_set_timeout (camera->port, timeout);
 		break;
 	default:
@@ -508,8 +517,14 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 	}
 	if (result < 0) {
 		gp_log (GP_LOG_DEBUG, "ptp2/usb_event", "reading event an error %d occurred", result);
+		if (result == GP_ERROR_TIMEOUT)
+			return PTP_ERROR_TIMEOUT;
 		return PTP_ERROR_IO;
 	}
+	if (result == 0) {
+		gp_log (GP_LOG_DEBUG, "ptp2/usb_event", "reading event an 0 read occurred, assuming timeout.");
+		return PTP_ERROR_TIMEOUT;
+	}
 	rlen = result;
 	if (rlen < 8) {
 		gp_log (GP_LOG_ERROR, "ptp2/usb_event",
@@ -536,12 +551,13 @@ ptp_usb_event (PTPParams* params, PTPContainer* event, int wait)
 	}
 	/* if we read anything over interrupt endpoint it must be an event */
 	/* build an appropriate PTPContainer */
-	event->Code=dtoh16(usbevent.code);
+	event->Nparam  = (rlen-12)/4;
+	event->Code   = dtoh16(usbevent.code);
 	event->SessionID=params->session_id;
 	event->Transaction_ID=dtoh32(usbevent.trans_id);
-	event->Param1=dtoh32(usbevent.param1);
-	event->Param2=dtoh32(usbevent.param2);
-	event->Param3=dtoh32(usbevent.param3);
+	event->Param1 = dtoh32(usbevent.param1);
+	event->Param2 = dtoh32(usbevent.param2);
+	event->Param3 = dtoh32(usbevent.param3);
 	return PTP_RC_OK;
 }
 
