@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -422,20 +422,18 @@ out:
  * with it and sometimes refuses to send data the correct way
  */
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
+put_file_func (CameraFilesystem *fs, const char *folder, const char *fn, CameraFile *file,
 	       void *data, GPContext *context)
 {
 	Camera *camera = data;
 	char *buf = NULL, *reply = NULL, *cmd =NULL;
-	const char *fn = NULL, *imgdata = NULL;
+	const char *imgdata = NULL;
 	int ret, channel, len;
 	long size;
 
 	ret = g3_cwd_command (camera->port, folder);
 	if (ret < GP_OK) goto out;
 
-	ret = gp_file_get_name (file, &fn);
-	if (ret < GP_OK) goto out;
 	ret = gp_file_get_data_and_size (file, &imgdata, &size);
 	if (ret < GP_OK) goto out;
 
@@ -628,10 +626,9 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	struct tm xtm;
 	char *cmd = NULL, *reply = NULL;
 
-	info->file.fields = GP_FILE_INFO_TYPE | GP_FILE_INFO_NAME |
+	info->file.fields = GP_FILE_INFO_TYPE |
 			GP_FILE_INFO_SIZE | GP_FILE_INFO_TYPE;
 	strcpy(info->file.type,GP_MIME_UNKNOWN);
-	strcpy(info->file.name, filename);
 
 	if (!strcmp(filename+9,"JPG") || !strcmp(filename+9,"jpg"))
 		strcpy(info->file.type,GP_MIME_JPEG);
@@ -646,6 +643,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		strcpy(info->file.type,"text/plain");
 
 	cmd = malloc(strlen("-FDAT ")+strlen(folder)+1+strlen(filename)+1);
+	if (!cmd) return GP_ERROR_NO_MEMORY;
 	sprintf(cmd, "-FDAT %s/%s", folder,filename);
 	ret = g3_ftp_command_and_reply(camera->port, cmd, &reply);
 	if (ret < GP_OK) goto out;
@@ -686,7 +684,7 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	}
 out:
 	if (reply) free(reply);
-	if (cmd) free(cmd);
+	free(cmd);
 
 	return (GP_OK);
 
@@ -790,7 +788,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 		int n = 0, channel, len, rlen;
 		ret = g3_channel_read(camera->port, &channel, &buf, &len); /* data. */
 		if (ret < GP_OK) goto out;
-		g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
+		ret = g3_channel_read(camera->port, &channel, &reply, &rlen); /* next reply  */
 		if (ret < GP_OK) goto out;
 		gp_log(GP_LOG_DEBUG, "g3" , "reply %s", reply);
 
@@ -811,14 +809,12 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 
 				/* we also get parts of fs info for free, so just set it */
 				info.file.fields =
-						GP_FILE_INFO_NAME |
 						GP_FILE_INFO_SIZE |
 						GP_FILE_INFO_MTIME;
 				info.file.size =(ubuf[n*32+28]<<24)|
 						(ubuf[n*32+29]<<16)|
 						(ubuf[n*32+30]<< 8)|
 						(ubuf[n*32+31]    );
-				strcpy(info.file.name,xfn);
 				if (!strcmp(xfn+9,"JPG") || !strcmp(xfn+9,"jpg")) {
 					strcpy(info.file.type,GP_MIME_JPEG);
 					info.file.fields |= GP_FILE_INFO_TYPE;
@@ -852,7 +848,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 					((year & 3) == 0 && month < 2 ? 1 : 0)+
 					3653);
 
-				ret = gp_filesystem_set_info_noop(fs, folder, info, context);
+				ret = gp_filesystem_set_info_noop(fs, folder, xfn, info, context);
  
 			}
 		}
