@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -24,7 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #include "crctab.h"
 
@@ -81,6 +81,11 @@
 	}						\
 }
 
+#ifndef HAVE_TM_GMTOFF
+/* required for time conversions in canon_int_set_time() */
+extern long int timezone;
+#endif
+
 static int
 ricoh_send (Camera *camera, GPContext *context, unsigned char cmd,
 	    unsigned char number,
@@ -317,6 +322,12 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 				       (ret_data[1] == 0x00))
 			break;
 
+		/* Check if the camera reported success, 2nd version. */
+		if ((*ret_len == 3) && (ret_data[0] == 0x00) &&
+				       (ret_data[1] == 0x06) &&
+				       (ret_data[2] == 0xff))
+			break;
+
 		/* If camera is busy, try again (but at most 4 times). */
 		if ((*ret_len == 3) && (ret_data[0] == 0x00) &&
 				       (ret_data[1] == 0x04) &&
@@ -366,7 +377,7 @@ ricoh_transmit (Camera *camera, GPContext *context, unsigned char cmd,
 
 int
 ricoh_get_pic_size (Camera *camera, GPContext *context, unsigned int n,
-		    unsigned long *size)
+		    uint64_t *size)
 {
 	unsigned char p[3], buf[0xff], len;
 
@@ -547,7 +558,7 @@ ricoh_disconnect (Camera *camera, GPContext *context)
 	return (GP_OK);
 }
 
-unsigned char header[236] = {
+static unsigned char header[236] = {
 	0x49,0x49,0x2a,0x00,0x08,0x00,0x00,0x00,0x0c,0x00,0x00,0x01,0x03,0x00,
 	0x01,0x00,0x00,0x00,0x50,0x00,0x00,0x00,0x01,0x01,0x03,0x00,0x01,0x00,
 	0x00,0x00,0x3c,0x00,0x00,0x00,0x02,0x01,0x03,0x00,0x03,0x00,0x00,0x00,
@@ -660,7 +671,11 @@ ricoh_set_date (Camera *camera, GPContext *context, time_t time)
 
 	/* Call localtime() to get 'extern long timezone' */
 	t = localtime (&time);
+#ifdef HAVE_TM_GMTOFF
+	time += t->tm_gmtoff;
+#else  
 	time += timezone;
+#endif 
 	t = localtime (&time);
 	GP_DEBUG ("ricoh_set_date: converted time to localtime %s "
 		  "(timezone is %ld)", asctime (t), timezone);
@@ -815,7 +830,7 @@ ricoh_put_file (Camera *camera, GPContext *context, const char *name,
 	if (strlen (name) > 12) {
 		gp_context_error (context, _("The filename's length must not "
 			"exceed 12 characters ('%s' has %i characters)."),
-			name, strlen (name));
+			name, (int)strlen (name));
 		return (GP_ERROR);
 	}
 
