@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -293,12 +293,11 @@ get_info_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		if (is_null (buffer))
 			continue;
 		if (!strcmp(buffer,filename)) {
-			info->file.fields	= GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT | GP_FILE_INFO_SIZE | GP_FILE_INFO_NAME;
+			info->file.fields	= GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT | GP_FILE_INFO_SIZE;
 			info->file.width	= 1024;
 			info->file.height	= 768;
 			strcpy(info->file.type,GP_MIME_JPEG);
-			strcpy(info->file.name,buffer);
-			sscanf(buffer+12,"%ld",&info->file.size);
+			sscanf(buffer+12,"%lld",&info->file.size);
 			return GP_OK;
 	        }
 		/* We are at the first item again */
