@@ -14,14 +14,14 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __LIBRARY_H__
 #define __LIBRARY_H__
 
-#include <gphoto2-camera.h>
+#include <gphoto2/gphoto2-camera.h>
 
 #define TIMEOUT	   2000
 
@@ -30,7 +30,7 @@
 	int res = (result);					\
 								\
 	if (res < 0) {						\
-		GP_DEBUG ("Operation failed (%i)!", res);	\
+		GP_DEBUG ("Operation failed in %s (%i)!", __FUNCTION__, res);	\
 		camera_stop (camera, context);			\
 		return (res);					\
 	}							\
@@ -41,7 +41,7 @@
 	int res = (result);					\
 								\
 	if (res < 0) {						\
-		GP_DEBUG ("Operation failed (%i)!", res);	\
+		GP_DEBUG ("Operation failed in %s (%i)!", __FUNCTION__, res);	\
 		camera_stop (camera, context);			\
 		free (camera->pl);				\
 		camera->pl = NULL;				\
@@ -54,14 +54,13 @@
 	int res = (result);					\
 								\
 	if (res < 0) {						\
-		GP_DEBUG ("Operation failed (%i)!", res);	\
+		GP_DEBUG ("Operation failed in %s (%i)!", __FUNCTION__, res);	\
 		free (camera->pl);				\
 		camera->pl = NULL;				\
 		return (res);					\
 	}							\
 }
 
-typedef enum _SierraAction SierraAction;
 enum _SierraAction {
 	SIERRA_ACTION_DELETE_LAST_PIC = 0x00,
 	SIERRA_ACTION_DELETE_ALL = 0x01,
@@ -75,6 +74,7 @@ enum _SierraAction {
 	SIERRA_ACTION_UPLOAD     = 0x0b,
 	SIERRA_ACTION_LCD_TEST   = 0x0c,
 };
+typedef enum _SierraAction SierraAction;
 
 typedef enum {
 	SIERRA_LOCKED_NO  = 0x00,
@@ -97,7 +97,6 @@ int sierra_set_locked (Camera *camera, unsigned int n, SierraLocked locked,
 		       GPContext *context);
 
 /* Communications functions */
-typedef enum _SierraSpeed SierraSpeed;
 enum _SierraSpeed {
 	SIERRA_SPEED_9600   = 1,
 	SIERRA_SPEED_19200  = 2,
@@ -105,6 +104,7 @@ enum _SierraSpeed {
 	SIERRA_SPEED_57600  = 4,
 	SIERRA_SPEED_115200 = 5
 };
+typedef enum _SierraSpeed SierraSpeed;
 int sierra_set_speed		  (Camera *camera, SierraSpeed speed,
 				   GPContext *context);
 int sierra_end_session		  (Camera *camera, GPContext *context);
