@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include <stdio.h>
 #include <_stdint.h>
@@ -38,7 +38,7 @@
  * the name/value union, and so it is the default value initialized.
  *
  * To init to a range, use:
- *	{ range: { 100.1, 2000.0, 10 } }, NULL 
+ *	{ .range = { 100.1, 2000.0, 10 } }, NULL 
  *
  * Casts are used to avoid warnings about using a const.
  *
@@ -153,7 +153,7 @@ static const RegisterDescriptorType cp880_reg_02[] = {
 #ifdef RANGE_FOR_SHUTTER
 static const ValueNameType cp880_reg_03_val_names[] = { 
 	{
-		{ range: { 0, 8000000 } }, NULL 
+		{ .range = { 0, 8000000 } }, NULL 
 	}
 };
 static const RegisterDescriptorType cp880_reg_03[] = { 
@@ -384,7 +384,7 @@ static const RegisterDescriptorType cp880_reg_34[] = {
  * Register 35: lcd brightness.
  */
 static const ValueNameType cp880_reg_35_val_names[] = {
-	{ { range: { 0, 7 } }, NULL },
+	{ { .range = { 0, 7 } }, NULL },
 };
 static const RegisterDescriptorType cp880_reg_35[] = { 
 	{
@@ -399,7 +399,7 @@ static const RegisterDescriptorType cp880_reg_35[] = {
  * what the maximum range should be.
  */
 static const ValueNameType cp880_reg_38_val_names[] = {
-	{ { range: { 0, 255 /* XXX? */ } }, NULL },
+	{ { .range = { 0, 255 /* XXX? */ } }, NULL },
 };
 static const RegisterDescriptorType cp880_reg_38[] = { 
 	{
@@ -450,7 +450,7 @@ static const RegisterDescriptorType cp880_reg_53[] = {
 #ifdef RANGE_FOR_EXPOSURE
 static const ValueNameType cp880_reg_69_val_names[] = {
 	{
-		{ range: { -2.0, 2.0, .1 } }, NULL,
+		{ .range = { -2.0, 2.0, .1 } }, NULL,
 	}
 };
 static const RegisterDescriptorType cp880_reg_69[] = { 
@@ -508,7 +508,7 @@ static const RegisterDescriptorType cp880_reg_70[] = {
  */
 static const ValueNameType cp880_reg_71_val_names[] = {
 	{
-		{ range: { 8.0, 20.0, .1 } }, NULL
+		{ .range = { 8.0, 20.0, .1 } }, NULL
 	}
 };
 static const RegisterDescriptorType cp880_reg_71[] = { 
@@ -555,7 +555,7 @@ static const RegisterDescriptorType cp880_reg_72[] = {
 	},
 	{ 
 		GP_WIDGET_RADIO, 0x00ff, 
-		"misc-exp-lenses", N_("Misc exposure/lense settings"), 
+		"misc-exp-lenses", N_("Misc exposure/lens settings"), 
 		VAL_NAME_INIT (cp880_reg_72_mask_lo_val_names)
 	},
 };
@@ -652,6 +652,39 @@ static CameraRegisterType cp2500_cam_regs[] = {
 	CAM_REG_TYPE_INIT (cp880, 53, 4, CAM_DESC_DEFAULT, 0), /* language */
 };
 
+/*
+ * All of the register used to modify picture settings. The register value
+ * received from the camera is stored into this data area, so it cannot be
+ * a const.
+ */
+static CameraRegisterType cp4300_pic_regs[] =  {
+	/* camera prefix, register number, size of register */
+	CAM_REG_TYPE_INIT (cp880, 01, 4, CAM_DESC_DEFAULT, 0), /* resolution/size */
+	CAM_REG_TYPE_INIT (cp880, 06, 4, CAM_DESC_DEFAULT, 0), /* color mode */
+	CAM_REG_TYPE_INIT (cp880, 07, 4, CAM_DESC_DEFAULT, 0), /* flash */
+	CAM_REG_TYPE_INIT (cp880, 19, 4, CAM_DESC_DEFAULT, 0), /* brightness/contrast */
+	CAM_REG_TYPE_INIT (cp880, 20, 4, CAM_DESC_DEFAULT, 0), /* white balance */
+#ifdef REG_30_FAILS
+	CAM_REG_TYPE_INIT (cp880, 30, 4, CAM_DESC_DEFAULT, 0), /* LED mode */
+#endif
+	CAM_REG_TYPE_INIT (cp880, 33, 4, CAM_DESC_DEFAULT, 0), /* focus mode */
+	CAM_REG_TYPE_INIT (cp880, 69, 8, CAM_DESC_DEFAULT, 0), /* exposure compensation */
+	CAM_REG_TYPE_INIT (cp880, 70, 4, CAM_DESC_DEFAULT, 0), /* exposure metering */
+	CAM_REG_TYPE_INIT (cp880, 71, 8, CAM_DESC_DEFAULT, 0), /* optical zoom */
+	CAM_REG_TYPE_INIT (cp880, 72, 4, CAM_DESC_DEFAULT, 0), /* digital zoom + lense +  AE lock */
+};
+
+/*
+ * All of the register used to modify camera settings.
+ */
+static CameraRegisterType cp4300_cam_regs[] = {
+	CAM_REG_TYPE_INIT (cp880, 02, 4, CAM_DESC_DEFAULT, 0), /* date-time */
+	CAM_REG_TYPE_INIT (cp880, 34, 4, CAM_DESC_SUBACTION, 
+			  SIERRA_ACTION_LCD_MODE), /* lcd mode */
+	CAM_REG_TYPE_INIT (cp880, 35, 4, CAM_DESC_DEFAULT, 0), /* LCD brightness */
+	CAM_REG_TYPE_INIT (cp880, 53, 4, CAM_DESC_DEFAULT, 0), /* language */
+};
+
 
 static const CameraRegisterSetType cp880_desc[] = {
 	{ 
@@ -675,6 +708,17 @@ static const CameraRegisterSetType cp2500_desc[] = {
 	},
 };
 
+static const CameraRegisterSetType cp4300_desc[] = {
+	{ 
+		N_("Picture Settings"), 
+		SIZE_ADDR (CameraRegisterType, cp4300_pic_regs)
+	},
+	{ 
+		N_("Camera Settings"), 
+		SIZE_ADDR (CameraRegisterType, cp4300_cam_regs)
+	},
+};
+
 static const char cp880_manual[] = 
 N_(
 "Nikon Coolpix 880:\n"
@@ -706,8 +750,10 @@ N_(
 );
 
 const CameraDescType cp880_cam_desc = { cp880_desc, cp880_manual,
-	SIERRA_EXT_PROTO, }; 
+	SIERRA_EXT_PROTO, };
 const CameraDescType cp995_cam_desc = { cp880_desc, cp995_manual,
 	SIERRA_EXT_PROTO, };
 const CameraDescType cp2500_cam_desc = { cp2500_desc, cp880_manual,
-	SIERRA_EXT_PROTO, }; 
+	SIERRA_EXT_PROTO, };
+const CameraDescType cp4300_cam_desc = { cp4300_desc, cp880_manual,
+	SIERRA_EXT_PROTO, };
