@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 /*
@@ -146,6 +146,7 @@ extern const CameraDescType oly3000z_cam_desc;
 extern const CameraDescType cp880_cam_desc;
 extern const CameraDescType cp995_cam_desc;
 extern const CameraDescType cp2500_cam_desc;
+extern const CameraDescType cp4300_cam_desc;
 
 /*
  * epson-desc.c:
