@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -106,7 +106,7 @@ web2_getexif(GPPort *port, GPContext *context, CameraFile *file)
     ret = web2_command(port, 1, WEB2_GET_EXIF, 0x00, 0, NULL, 0);
     if (ret!=GP_OK)
 	return ret;
-    gp_file_append(file, ExifHeader, sizeof(ExifHeader));
+    gp_file_append(file, (char*)ExifHeader, sizeof(ExifHeader));
     ret = gp_port_read(port, xbuf, sizeof(xbuf));
     if (ret < GP_OK) {
 	gp_file_clean(file);
@@ -131,7 +131,7 @@ web2_getthumb(GPPort *port, GPContext *context, CameraFile *file)
     ret = web2_command(port, 1, WEB2_GET_THUMBNAIL, 0x00, 0, NULL, 0);
     if (ret!=GP_OK)
 	return ret;
-    ret = gp_port_read(port, buf, sizeof(buf));
+    ret = gp_port_read(port, (char*)buf, sizeof(buf));
     if (ret < GP_OK)
 	return ret;
     for (i=0;i<ret;i+=2) {
@@ -139,7 +139,7 @@ web2_getthumb(GPPort *port, GPContext *context, CameraFile *file)
 	buf[i] = buf[i+1];
 	buf[i+1] = tmp;
     }
-    gp_file_append(file, buf, ret);
+    gp_file_append(file, (char*)buf, ret);
     return GP_OK;
 }
 
@@ -147,7 +147,7 @@ static int
 web2_get_file_info(GPPort *port, GPContext *context, char *name, int *filesize) {
     unsigned char cmdbuf[26];
     int i, hmm, ret;
-    ret = web2_command(port, 0, WEB2_GET_DIRENTRY, 0, 0, cmdbuf, 26);
+    ret = web2_command(port, 0, WEB2_GET_DIRENTRY, 0, 0, (char*)cmdbuf, 26);
 
     /* 0 usually? */
     hmm = cmdbuf[0]  | (cmdbuf[1] << 8);
@@ -555,8 +555,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	fprintf(stderr,"\n");
 	*/
 
-	gp_file_set_mime_type (file, "image/jpeg");
-	gp_file_set_name (file, filename);
+	gp_file_set_mime_type (file, GP_MIME_JPEG);
 
 	switch (type) {
 	case GP_FILE_TYPE_NORMAL:
