@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include <config.h>
 #include <stdio.h>
@@ -111,8 +111,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
    if (image_no < GP_OK)
      return image_no;
 
-   gp_file_set_name (file, filename);
-
    switch (type) {
     case GP_FILE_TYPE_NORMAL:
       result = ultrapocket_getpicture(camera,context,&data,&size,filename);
