@@ -14,9 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
+
+#define _BSD_SOURCE
+
 #include <config.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,6 +37,7 @@
 
 #define GP_MODULE "Smal Ultrapocket"
 
+#include <locale.h>
 #ifdef ENABLE_NLS
 #  include <libintl.h>
 #  undef _
@@ -228,7 +232,7 @@ ultrapocket_getrawpicture(Camera *camera, GPContext *context,
 	imgstart = 0x29;
 	break;
      default:
-	break;
+        return GP_ERROR;
     }
 
    tile = BAYER_TILE_BGGR;
