@@ -14,10 +14,12 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include <config.h>
 
 #include <stdlib.h>
@@ -25,7 +27,6 @@
 #include <string.h>
 
 #include <bayer.h>
-#include <gamma.h>
 
 #include <gphoto2/gphoto2.h>
 
@@ -97,10 +98,10 @@ camera_abilities (CameraAbilitiesList *list)
 			a.operations = GP_OPERATION_NONE;
 		else
 			a.operations = GP_OPERATION_CAPTURE_IMAGE;
-			a.folder_operations = GP_FOLDER_OPERATION_DELETE_ALL;
-			a.file_operations   = GP_FILE_OPERATION_DELETE|
-					GP_FILE_OPERATION_PREVIEW;
-			gp_abilities_list_append (list, a);
+		a.folder_operations = GP_FOLDER_OPERATION_DELETE_ALL;
+		a.file_operations   = GP_FILE_OPERATION_DELETE|
+				GP_FILE_OPERATION_PREVIEW;
+		gp_abilities_list_append (list, a);
 	}
 	return GP_OK;
 }
@@ -194,7 +195,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	unsigned char *data = NULL; 
 	unsigned char  *ppm, *ptr, *avi=NULL, *frame_data = NULL;
 	unsigned char *p_data = NULL, *p_buf = NULL;
-	unsigned char gtable[256];
 	unsigned int numframes = 0;
 	unsigned int endpoint, bytes_used = 0;
 	unsigned int framestart[1024];
@@ -204,7 +204,6 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	unsigned int CAM_OFFSET;
 	unsigned char POST_CODE;
 	unsigned char compressed=0;
-	char name[16];
 	int ret = 0;
 
 	GP_DEBUG ("Downloading pictures!\n");
@@ -241,6 +240,8 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	GP_DEBUG( "avitype is %d\n", avitype);
 	GP_DEBUG ("height of picture %i is %i\n", k+1,h);
 	rawsize = sonix_read_data_size (camera->port, k);
+	if (rawsize < GP_OK)
+		return rawsize;
 	GP_DEBUG("rawsize = 0x%x = %i\n", rawsize, rawsize);
 	if(rawsize%0x40) 
 		buffersize = rawsize - (rawsize%0x40) + 0x40;
@@ -297,17 +298,15 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			GP_DEBUG("CAM_OFFSET=%i\n",CAM_OFFSET);
 			frame_size = w*h;
 			gp_file_set_mime_type (file, GP_MIME_AVI);
-			gp_file_set_name (file, name); 
 			frame_data = malloc(frame_size);
 			if (!frame_data) {
-				free (frame_data);
 				return -1;
 			}
 			size = (numframes)*(3*frame_size+ 8) + 224;
 			GP_DEBUG("size = %i\n", size);
 			avi = malloc(224);
 			if (!avi) {
-				free(avi); 
+				free(frame_data);
 				return -1;
 			}
 			GP_DEBUG("avi malloc'ed\n");
@@ -330,7 +329,8 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			avi[0x8c] = numframes&0xff;
 			ptr=malloc(3*frame_size+8);
 			if(!ptr) {
-				free(ptr);
+				free(frame_data);
+				free(avi);
 				return GP_ERROR_NO_MEMORY;
 			}
 			GP_DEBUG("avi hdr written\n");
@@ -359,29 +359,28 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 					sonix_decode (frame_data, 
 					data+offset+CAM_OFFSET, w, h); 
 					sonix_cols_reverse(frame_data, w, h);
-					gp_bayer_decode(frame_data, w, h, ptr+
-					    SAKAR_AVI_FRAME_HEADER_LENGTH, 
-							    BAYER_TILE_GBRG);
+                                        gp_ahd_decode(frame_data, w, h, ptr+
+                                            SAKAR_AVI_FRAME_HEADER_LENGTH,
+                                                            BAYER_TILE_GBRG);
 					break;
 				case DECOMP:
 					sonix_decode (frame_data, 
 					data+offset+CAM_OFFSET, w, h); 
 					sonix_rows_reverse(frame_data, w, h);
-					gp_bayer_decode(frame_data, w, h, ptr+
-					    SAKAR_AVI_FRAME_HEADER_LENGTH, 
-							    BAYER_TILE_GRBG);
+                                        gp_ahd_decode(frame_data, w, h, ptr+
+                                            SAKAR_AVI_FRAME_HEADER_LENGTH,
+                                                            BAYER_TILE_GRBG);
 					break;
 				default:
 					memcpy(frame_data, data + offset
 					    +CAM_OFFSET, frame_size);
 					sonix_rows_reverse(frame_data, w, h);
-					gp_bayer_decode(frame_data, w, h, ptr+
-					    SAKAR_AVI_FRAME_HEADER_LENGTH, 
-							    BAYER_TILE_GRBG);
+                                        gp_ahd_decode(frame_data, w, h, ptr+
+                                            SAKAR_AVI_FRAME_HEADER_LENGTH,
+                                                            BAYER_TILE_GRBG);
 				}
-				gp_gamma_fill_table(gtable, .65);
-				gp_gamma_correct_single(gtable, ptr+
-				    SAKAR_AVI_FRAME_HEADER_LENGTH, w * h);
+				white_balance(ptr+SAKAR_AVI_FRAME_HEADER_LENGTH, 
+						w * h, 1.2);
 				gp_file_append(file, (char *)ptr, 
 				    3*frame_size+ 
 						SAKAR_AVI_FRAME_HEADER_LENGTH);
@@ -424,33 +423,25 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			if (camera->pl->post&REVERSE) {
 			sonix_decode (p_data, data+CAM_OFFSET, w, h);
 			sonix_byte_reverse(p_data, w*h);
-			gp_bayer_decode (p_data, w, h, ptr, 
-						    BAYER_TILE_BGGR);
+                        gp_ahd_decode(p_data, w, h, ptr, BAYER_TILE_BGGR);
 			break;
 		case DECOMP:
 			sonix_decode (p_data, data+CAM_OFFSET, w, h);
-			gp_bayer_decode (p_data, w, h, ptr, 
-						    BAYER_TILE_RGGB);
+                        gp_ahd_decode(p_data, w, h, ptr, BAYER_TILE_RGGB);
 			break;
 		default:
 			memcpy (p_data, data+CAM_OFFSET, w*h);
-			gp_bayer_decode (p_data, w, h, ptr, BAYER_TILE_RGGB);
+                        gp_ahd_decode(p_data, w, h, ptr, BAYER_TILE_RGGB);
 		}
 		free (p_data);
-		gp_gamma_fill_table(gtable, .65);
-		gp_gamma_correct_single(gtable, ptr, w * h);
+		white_balance(ptr, w * h, 1.2);
+		GP_DEBUG("white_balance run on photo number %03d \n", k+1);
 		gp_file_set_mime_type (file, GP_MIME_PPM);
-		gp_file_set_name (file, filename); 
 		gp_file_set_data_and_size (file, (char *)ppm, size);
 		free (data);
 		return GP_OK;
 	case GP_FILE_TYPE_RAW: 
-#if 0 /* MARCUS: likely bad, comment out for now */
-				/* Save the fwversion code in the data */
-				data[endpoint-10]=camera->pl->fwversion[1];
-#endif
 		gp_file_set_mime_type(file, GP_MIME_RAW);
-		gp_file_set_name(file, filename);
 		gp_file_append( file, (char *)data, rawsize);
 		free(data);
 		GP_DEBUG("rawsize= 0x%x = %i\n", rawsize, rawsize);
@@ -497,9 +488,9 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	if (k+1 != camera->pl->num_pics) {
 		GP_DEBUG("Only the last photo can be deleted!\n");
 		return GP_ERROR_NOT_SUPPORTED;
-	} else
-	    sonix_delete_last (camera->port);
-	    camera->pl->num_pics -= 1;
+	}
+	sonix_delete_last (camera->port);
+	camera->pl->num_pics -= 1;
 	return GP_OK;
 }
 
@@ -548,11 +539,8 @@ camera_exit (Camera *camera, GPContext *context)
 
 static CameraFilesystemFuncs fsfuncs = {
 	.file_list_func = file_list_func,
-	.folder_list_func = NULL,
-	.get_info_func = NULL,
 	.get_file_func = get_file_func,
 	.del_file_func = delete_file_func,
-	.put_file_func = NULL, 
 	.delete_all_func = delete_all_func,
 };
 
