@@ -32,7 +32,7 @@
 
 #define MAX_PICTURE_NUM 200
 static unsigned char  picture_index[MAX_PICTURE_NUM];
-static u_short picture_thumbnail_index[MAX_PICTURE_NUM];
+static unsigned short picture_thumbnail_index[MAX_PICTURE_NUM];
 static unsigned char  picture_protect[MAX_PICTURE_NUM];
 static unsigned char  picture_rotate[MAX_PICTURE_NUM];
 
@@ -306,6 +306,7 @@ get_thumbnail(GPPort *port,char *name, CameraFile *file, int format, int n)
   return gp_file_append (file, &buf[256], filelen);
 }
 
+#if 0
 static void
 get_date_info(GPPort *port, char *name, char *outfilename ,char *newfilename)
 {
@@ -388,6 +389,7 @@ get_date_info(GPPort *port, char *name, char *outfilename ,char *newfilename)
   *q = 0;
 
 }
+#endif
 
 static int
 get_picture(GPPort *port, int n, CameraFile *file, int format, int ignore, int all_pic_num,
@@ -499,8 +501,7 @@ static int get_file_func (CameraFilesystem *fs, const char *folder,
         if(!F1ok(camera->port))
            return (GP_ERROR);
 
-	gp_file_set_name (file, filename);
-	gp_file_set_mime_type (file, "image/jpeg");
+	gp_file_set_mime_type (file, GP_MIME_JPEG);
 
         /* Retrieve the number of the photo on the camera */
 	num = gp_filesystem_number(camera->fs, "/", filename, context);
