@@ -19,9 +19,10 @@
 /*                                                              */
 /* You should have received a copy of the GNU Library General   */
 /* Public License along with this library; if not, write to the */
-/* Free Software Foundation, Inc., 59 Temple Place - Suite 330, */
-/* Boston, MA 02111-1307, USA.                                  */
+/* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,*/
+/* Boston, MA  02110-1301  USA					*/
 /****************************************************************/
+#define _BSD_SOURCE
 
 #include "config.h"
 
@@ -287,14 +288,6 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 	return (GP_OK);
 }
 
-static int
-camera_manual (Camera *camera, CameraText *about, GPContext *context)
-{
-	return (GP_OK);
-}
-
-
-
 static int
 camera_about (Camera *camera, CameraText *about, GPContext *context)
 {
@@ -414,14 +407,9 @@ get_file_func (CameraFilesystem *fs, const char *folder,
 		default:
 			return GP_ERROR_NOT_SUPPORTED;
 	}
-
 	if (!data)
 		return GP_ERROR;
-
-	CHECK (gp_file_set_data_and_size (file, data, size));
-	CHECK (gp_file_set_name (file, filename));
-
-	return GP_OK;
+	return gp_file_set_data_and_size (file, data, size);
 }
 
 static int
@@ -446,9 +434,6 @@ get_info_func (CameraFilesystem *fs, const char *folder,
 	if (n < flash_file_count) {
 		CHECK (spca50x_flash_get_file_name(camera->pl,
 					n, name));
-		strncpy (info->file.name, name,
-				sizeof (info->file.name));
-
 		CHECK (spca50x_flash_get_file_dimensions(
 					camera->pl, n, &w, &h));
 		strcpy (info->file.type, GP_MIME_JPEG);
@@ -460,7 +445,6 @@ get_info_func (CameraFilesystem *fs, const char *folder,
 	if (cam_has_sdram (camera->pl) && n >= flash_file_count ){
 		CHECK (spca50x_sdram_get_file_info (camera->pl,
 					n-flash_file_count, &file));
-		strncpy (info->file.name, filename, sizeof (info->file.name));
 		if (file->mime_type == SPCA50X_FILE_TYPE_IMAGE) {
 			strcpy (info->file.type, GP_MIME_JPEG);
 			info->preview.width = 160;
@@ -475,7 +459,7 @@ get_info_func (CameraFilesystem *fs, const char *folder,
 
 	}
 	info->file.fields =
-		GP_FILE_INFO_NAME | GP_FILE_INFO_TYPE
+		GP_FILE_INFO_TYPE
 		| GP_FILE_INFO_WIDTH | GP_FILE_INFO_HEIGHT;
 
 	info->file.mtime = 0;
@@ -502,6 +486,9 @@ delete_file_func (CameraFilesystem *fs, const char *folder,
 	if (cam_has_flash(camera->pl) || cam_has_card(camera->pl) ) {
 		CHECK (spca50x_flash_get_filecount
 					(camera->pl, &flash_file_count));
+	} else {
+		/* should not happen really */
+		return GP_ERROR;
 	}
 	if (n < flash_file_count) {
 		return spca500_flash_delete_file (camera->pl, n);
@@ -557,7 +544,6 @@ camera_init (Camera *camera, GPContext *context)
 	/* First, set up all the function pointers */
 	camera->functions->exit = camera_exit;
 	camera->functions->summary = camera_summary;
-	camera->functions->manual = camera_manual;
 	camera->functions->about = camera_about;
 	camera->functions->capture = camera_capture;
 
@@ -601,7 +587,8 @@ camera_init (Camera *camera, GPContext *context)
 			int same;
 			char *m = strdup( models[x].model );
 			char *p = strchr (m, ':' );
-			*p = ' ';
+
+			if (p) *p = ' ';
 			same = !strcmp (m, abilities.model);
 			free (m);
 			if (same) {
@@ -627,7 +614,7 @@ camera_init (Camera *camera, GPContext *context)
 
 	if ((camera->pl->bridge == BRIDGE_SPCA504) ||
 	    (camera->pl->bridge == BRIDGE_SPCA504B_PD)) {
-//		if (abilities.usb_vendor != 0x04fc && abilities.usb_product != 0x504a )
+/*		if (abilities.usb_vendor != 0x04fc && abilities.usb_product != 0x504a ) */
 		if (!(abilities.usb_vendor == 0x04fc && abilities.usb_product == 0x504a ))
 			ret = spca50x_reset (camera->pl);
 	}
