@@ -13,9 +13,11 @@
  * GNU Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the 
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
+#define _BSD_SOURCE
 #include "config.h"
 
 #include <string.h>
@@ -23,7 +25,7 @@
 #ifdef HAVE_ICONV
 # include <langinfo.h>
 #endif
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 # include <gd.h>
 #endif
 
@@ -128,7 +130,7 @@ static int get_file_idx(CameraPrivateLibrary *pl, const char *folder,
 	return i;
 }
 
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 static void
 rotate90 (gdImagePtr src, gdImagePtr dest)
 {
@@ -179,7 +181,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 {
 	Camera *camera = data;
 	int idx, size;
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	int ret;
 	gdImagePtr im, rotated;
 	void *gdpng;
@@ -202,7 +204,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 		return GP_OK;
 	}
 
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	if (type != GP_FILE_TYPE_NORMAL)
 		return GP_ERROR_NOT_SUPPORTED;
 
@@ -248,13 +250,12 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 }
 
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
-	       void *data, GPContext *context)
+put_file_func (CameraFilesystem *fs, const char *folder, const char *name, 
+	CameraFileType type, CameraFile *file, void *data, GPContext *context)
 {
-#ifdef HAVE_GD
+#ifdef HAVE_LIBGD
 	Camera *camera = data;
 	char *c, *in_name, *out_name, *filedata = NULL;
-	const char *name;
 	int ret, in_width, in_height, in_x, in_y;
 	size_t inc, outc;
 	double aspect_in, aspect_out;
@@ -269,8 +270,6 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 	if (strcmp (folder, "/"))
 		return GP_ERROR_DIRECTORY_NOT_FOUND;
 
-	gp_file_get_name (file, &name);
-
 	inc = strlen (name);
 	in_name = strdup (name);
 	outc = inc;
