@@ -14,14 +14,14 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef LIBRARY_H
 #define LIBRARY_H
 
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 int stv0674_ping              (GPPort *);
 int stv0674_summary           (GPPort *, char*);
