@@ -13,8 +13,9 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -117,7 +118,7 @@ static int stv0680_cmd(GPPort *port, unsigned char cmd,
 	return GP_OK;
 }
 
-static int stv0680_try_cmd(GPPort *port, unsigned char cmd,
+int stv0680_try_cmd(GPPort *port, unsigned char cmd,
 		unsigned short data,
 		unsigned char *response, unsigned char response_len
 ) {
@@ -262,9 +263,16 @@ int stv0680_get_image(GPPort *port, int image_no, CameraFile *file)
 
 	data = malloc(size * 3);
 	tmpdata1 = malloc(size * 3);
-	if (!tmpdata1) return GP_ERROR_NO_MEMORY;
+	if (!tmpdata1) {
+		free (data);
+		return GP_ERROR_NO_MEMORY;
+	}
 	tmpdata2 = malloc(size * 3);
-	if (!tmpdata2) return GP_ERROR_NO_MEMORY;
+	if (!tmpdata2) {
+		free (data);
+		free (tmpdata1);
+		return GP_ERROR_NO_MEMORY;
+	}
 	gp_bayer_expand (raw, w, h, tmpdata1, BAYER_TILE_GBRG_INTERLACED);
 	light_enhance(w,h,coarse,imghdr.avg_pixel_value,fine,tmpdata1);
 	/* gp_bayer_interpolate (tmpdata1, w, h, BAYER_TILE_GBRG_INTERLACED); */
@@ -486,8 +494,10 @@ int stv0680_capture_preview(GPPort *port, char **data, int *size)
 		break;
 	default:break;
 	}
-	if ((ret = stv0680_try_cmd(port, CMDID_STOP_VIDEO, 0, NULL, 0)!=GP_OK))
+	if ((ret = stv0680_try_cmd(port, CMDID_STOP_VIDEO, 0, NULL, 0)!=GP_OK)) {
+		free (raw);
 		return ret;
+	}
 	sprintf(header, "P6\n# gPhoto2 stv0680 image\n%d %d\n255\n", w, h);
 	*data = malloc((*size * 3) + strlen(header));
 	strcpy(*data, header);
@@ -514,7 +524,7 @@ int stv0680_summary(GPPort *port, char *txt)
     struct stv680_image_info imginfo;
     int ret;
 
-    strcpy(txt,_("Information for STV0680 based camera:\n"));
+    strcpy(txt,_("Information on STV0680-based camera:\n"));
     /* Get Camera Information */
     if ((ret = stv0680_try_cmd(port, CMDID_GET_CAMERA_INFO,
 				0, (void*)&caminfo, sizeof(caminfo)) < 0))
@@ -533,7 +543,7 @@ int stv0680_summary(GPPort *port, char *txt)
     );
     /* HWCONFIG_COMMSLINK_MASK ... not really needed, the user knows, he
      * plugged it in. */
-    sprintf(txt+strlen(txt),_("Camera is configured for lights flickering with %dHz.\n"),
+    sprintf(txt+strlen(txt),_("Camera is configured for lights flickering by %dHz.\n"),
     	(caminfo.hardware_config & HWCONFIG_FLICKERFREQ_60HZ)?60:50
     );
     sprintf(txt+strlen(txt),_("Memory in camera: %d Mbit.\n"),
