@@ -13,14 +13,15 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   
+ * along with this program; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef LIBRARY_H
 #define LIBRARY_H
 
-#include <gphoto2-port.h>
+#include <gphoto2/gphoto2-port.h>
 
 int stv0680_ping              (GPPort *);
 int stv0680_summary           (GPPort *, char*);
