@@ -13,8 +13,9 @@
  * GNU General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA   
+ * along with this program; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef STV0680_H
@@ -165,4 +166,9 @@ struct stv680_user_info {
 #define STV0680_VGA_WIDTH	644
 #define STV0680_VGA_HEIGHT	484
 
+int stv0680_try_cmd(GPPort *port, unsigned char cmd,
+                unsigned short data,
+                unsigned char *response, unsigned char response_len);
+
+
 #endif
