@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -22,8 +22,8 @@
 #include <stdio.h>
 #include <string.h>
 
-#include <gphoto2-library.h>
-#include <gphoto2-port-log.h>
+#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-port-log.h>
 
 #include "sx330z.h"
 
@@ -67,7 +67,7 @@
  *   please report if you have others that are working
  */
  
-static struct
+static const struct
 {
  char *model;
  int usb_vendor;
@@ -141,6 +141,7 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 	for (pcnt = 0; pcnt < tpages; pcnt++) {
 		CR (sx330z_get_toc_page (camera, context, &toc, pcnt));
 		for (ecnt = 0; ecnt < toc.numEntries; ecnt++) {
+			char fn[20];
 
 			info.audio.fields = GP_FILE_INFO_NONE;
 			info.preview.fields = GP_FILE_INFO_TYPE;
@@ -152,16 +153,14 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 			info.file.permissions = GP_FILE_PERM_READ |
 						GP_FILE_PERM_DELETE;
 			strcpy (info.file.type,GP_MIME_JPEG); 
-			sprintf (info.file.name, "%.12s",
-				 toc.entries[ecnt].name);
+			sprintf (fn, "%.12s", toc.entries[ecnt].name);
 
 			/*
 			 * Append directly to the filesystem instead of to
 			 * the list, because we have additional information.
 			 */
-			gp_filesystem_append (camera->fs, folder,
-					      info.file.name, context);
-			gp_filesystem_set_info_noop (camera->fs, folder,
+			gp_filesystem_append (camera->fs, folder, fn, context);
+			gp_filesystem_set_info_noop (camera->fs, folder, fn,
 						     info, context);
 		}
 		gp_context_progress_update (context, id, pcnt);
@@ -234,16 +233,6 @@ camera_id(CameraText *id)
 
 
 
-/*
- * Camera Summary 
- */
-static int 
-camera_summary(Camera *camera,CameraText *summary,GPContext *context)
-{
- return(GP_ERROR_NOT_SUPPORTED);
-} /* camera summary */
-
-
 /*
  * Camera about 
  */
@@ -289,7 +278,6 @@ camera_init(Camera *camera,GPContext *context)
  /*CR(gp_port_get_settings(camera->port,&settings));*/
  
  camera->functions->about=camera_about; 
- camera->functions->summary=camera_summary; 
  camera->functions->exit=camera_exit;
  gp_port_get_settings(camera->port,&settings);
  if (camera->port->type!=GP_PORT_USB)
