@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "sx330z.h"
@@ -29,10 +29,10 @@
 /* strncmp */
 #include <string.h>
 
-#include <gphoto2-endian.h>
+#include "gphoto2-endian.h"
 
 /*convert to correct endianness */
-int 
+static int
 sx330z_fill_req(int8_t *buf, struct traveler_req *req)
 {
  *((int16_t*)buf) = htole16(req->always1);     buf += 2;
@@ -46,7 +46,7 @@ sx330z_fill_req(int8_t *buf, struct traveler_req *req)
 } /* sx330z_fill_req */
 
 /*convert to correct endianness */
-int 
+static int
 sx330z_fill_ack(int8_t *buf, struct traveler_ack *ack)
 {
  ack->always3 = le32toh(*(int32_t*)buf);   buf += 4;
@@ -57,7 +57,7 @@ sx330z_fill_ack(int8_t *buf, struct traveler_ack *ack)
 } /* sx330z_fill_ack */
 
 /*convert to correct endianness */
-int 
+static int
 sx330z_fill_toc_page(int8_t *buf, struct traveler_toc_page *toc)
 {
  int cnt;
@@ -93,7 +93,7 @@ sx330z_init(Camera *camera, GPContext *context)
 /*
  * Read block described by req 
  */
-int 
+static int 
 sx330z_read_block(Camera *camera, GPContext *context, struct traveler_req *req, char *buf)
 {
  int ret;
