@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 /*
  * GPhoto library for sx330z camera ....
@@ -23,8 +23,8 @@
  */
 
 
-#include <gphoto2-camera.h>
-#include <gphoto2-context.h>
+#include <gphoto2/gphoto2-camera.h>
+#include <gphoto2/gphoto2-context.h>
 
 
 /* USB spec violating ! */
@@ -100,11 +100,6 @@ struct traveler_ack
 };
 
 
-/* endianness handling */
-int sx330z_fill_req(int8_t *buf,struct traveler_req *req);
-int sx330z_fill_ack(int8_t *buf,struct traveler_ack *ack);
-int sx330z_fill_toc_page(int8_t *buf,struct traveler_toc_page *toc);
-
 /*
  *  There's not a real initialization,
  *  but ...
@@ -118,11 +113,6 @@ int sx330z_init(Camera *camera,GPContext *context);
 int sx330z_get_toc_num_pages(Camera *camera,GPContext *context, int32_t *pages);
 
 
-/*
- * Read a data-block  (TOC / Image)
- */
-int sx330z_read_block(Camera *camera,GPContext *context,struct traveler_req *req,char *buf);
-
 /*
  * Get TOC 
  */
