@@ -15,8 +15,9 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with puppy; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with puppy; if not, write to the 
+  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA  02110-1301  USA
 
 */
 
@@ -27,14 +28,15 @@
 #include <sys/types.h>
 
 /* Encapsulation of MJD date and h:m:s timestamp */
+#pragma pack(1)
 struct tf_datetime
 {
     unsigned short mjd;
     unsigned char hour;
     unsigned char minute;
     unsigned char second;
-} __attribute__ ((packed));
-
+};
+#pragma pack()
 
 time_t tfdt_to_time(struct tf_datetime *dt);
 void time_to_tfdt(time_t t, struct tf_datetime *dt);
