@@ -16,11 +16,12 @@
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
-  along with puppy; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+  along with puppy; if not, write to the 
+  Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA  02110-1301  USA
 
 */
-
+#define _BSD_SOURCE
 
 #include "config.h"
 
@@ -137,7 +138,7 @@ strdup_to_locale (char *str) {
 		if (ret == -1) {
 			perror("iconv");
 			free (dest);
-			dest = NULL;
+			return NULL;
 		}
 		break;
 	}
@@ -426,24 +427,23 @@ decode_and_get_info(Camera *camera, const char *folder, struct tf_packet *p, con
 			name = _convert_and_logname (camera, (char*)entries[i].name);
 			if (!strcmp (name, fn)) { /* the wanted current one */
 				memset (info, 0, sizeof (*info));
-				info->file.fields = GP_FILE_INFO_NAME|GP_FILE_INFO_SIZE|GP_FILE_INFO_MTIME;
+				info->file.fields = GP_FILE_INFO_SIZE|GP_FILE_INFO_MTIME;
 				if (strstr (name, ".rec")) {
 					info->file.fields |= GP_FILE_INFO_TYPE;
 					strcpy (info->file.type, GP_MIME_MPEG);
 				}
-				strcpy (info->file.name, name);
 				info->file.size = get_u64(&entries[i].size);
 				info->file.mtime = tfdt_to_time(&entries[i].stamp);
 			} else { /* cache the others to avoid further turnarounds */
 				CameraFileInfo	xinfo;
 
 				memset (&xinfo, 0, sizeof (xinfo));
-				xinfo.file.fields = GP_FILE_INFO_NAME|GP_FILE_INFO_TYPE|GP_FILE_INFO_SIZE|GP_FILE_INFO_MTIME;
+				xinfo.file.fields = GP_FILE_INFO_TYPE|GP_FILE_INFO_SIZE|GP_FILE_INFO_MTIME;
 				strcpy (xinfo.file.type, GP_MIME_MPEG);
-				strcpy (xinfo.file.name, name);
 				xinfo.file.size = get_u64(&entries[i].size);
 				xinfo.file.mtime = tfdt_to_time(&entries[i].stamp);
-				gp_filesystem_set_info_noop (camera->fs, folder, xinfo, context);
+				gp_filesystem_append (camera->fs, folder, name, context); /* FIXME: might fail if exist? */
+				gp_filesystem_set_info_noop (camera->fs, folder, name, xinfo, context);
 			}
 			break;
 		default:
@@ -719,14 +719,13 @@ out:
 
 #if 0
 static int
-put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
+put_file_func (CameraFilesystem *fs, const char *folder, const char *filename, CameraFile *file,
 	       void *data, GPContext *context)
 {
 	Camera *camera = data;
 
 	/*
-	 * Upload the file to the camera. Use gp_file_get_data_and_size,
-	 * gp_file_get_name, etc.
+	 * Upload the file to the camera. Use gp_file_get_data_and_size etc.
 	 */
 	int result = -EPROTO;
 	time_t startTime = time(NULL);
@@ -746,10 +745,6 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file,
 	char *path;
 	struct tf_packet reply;
 
-	r = gp_file_get_name (file, &filename);
-	if (r < GP_OK)
-		return r;
-
 	if(0 != fstat(src, &srcStat))
 	{
 		gp_log (GP_LOG_ERROR, "topfield", "ERROR: Can not examine source file: %s\n",
