@@ -12,8 +12,8 @@
  * owner to the string passed on the cmdline.
  *
  * Same as:
- *	gphoto2 --get-config owner
- *	gphoto2 --set-config owner="Owner Name"
+ *	gphoto2 --get-config ownername
+ *	gphoto2 --set-config ownername="Owner Name"
  */
 
 int main(int argc, char **argv) {
@@ -37,14 +37,14 @@ int main(int argc, char **argv) {
 		return 0;
 	}
 
-	ret = get_config_value_string (camera, "owner", &owner, context);
+	ret = get_config_value_string (camera, "ownername", &owner, context);
 	if (ret < GP_OK) {
 		printf ("Could not query owner.\n");
 		goto out;
 	}
 	printf("Current owner: %s\n", owner);
 	if (argc > 1) {
-		ret = set_config_value_string (camera, "owner", argv[1], context);
+		ret = set_config_value_string (camera, "ownername", argv[1], context);
 		if (ret < GP_OK) {
 			fprintf (stderr, "Failed to set camera owner to %s; %d\n", argv[1], ret);
 		} else 
