@@ -17,13 +17,16 @@
  * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 #include <gphoto2/gphoto2-context.h>
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -158,19 +161,54 @@ gp_context_progress_start (GPContext *context, float target,
 {
 	va_list args;
 	unsigned int id;
+#ifdef HAVE_VA_COPY
+	va_list xargs;
+#else
+#define xargs args
+#endif
+	int strsize = 1000;
+	char *str;
+	int n;
 
 	if (!context)
 		return (0);
+	if (!context->progress_start_func)
+		return (0);
 
-	if (context->progress_start_func) {
-		va_start (args, format);
-		id = context->progress_start_func (context, target, format,
-					args, context->progress_func_data);
-		va_end (args);
-		return (id);
+	str = malloc(strsize);
+	if (!str) {
+		return 0;
 	}
+	va_start (args, format);
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#endif
+	n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+	va_end (xargs);
+#endif
+	if (n+1>strsize) {
+		free (str);
+		str = malloc(n+1);
+		if (!str) { 
+			va_end (args);
+			return 0;
+		}
+		strsize = n+1;
+#ifdef HAVE_VA_COPY
+		va_copy (xargs, args);
+#endif
+		n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+		va_end (xargs);
+#endif
+	}
+	va_end (args);
 
-	return (0);
+	id = context->progress_start_func (context, target, str,
+				context->progress_func_data);
+	free (str);
+	return (id);
 }
 
 void
@@ -199,42 +237,98 @@ void
 gp_context_error (GPContext *context, const char *format, ...)
 {
 	va_list args;
+#ifdef HAVE_VA_COPY
+	va_list xargs;
+#endif
+	int strsize = 1000;
+	char *str;
+	int n;
 
-	/* Log the error message */
+	str = malloc(strsize);
+	if (!str) {
+		return;
+	}
 	va_start (args, format);
-	gp_logv (GP_LOG_ERROR, "context", format, args);
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#endif
+	n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+	va_end (xargs);
+#endif
+	if (n+1>strsize) {
+		free (str);
+		str = malloc(n+1);
+		if (!str) { 
+			va_end (args);
+			return;
+		}
+		strsize = n+1;
+#ifdef HAVE_VA_COPY
+		va_copy (xargs, args);
+#endif
+		n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+		va_end (xargs);
+#endif
+	}
 	va_end (args);
 
-	if (!context)
-		return;
+	/* Log the error message */
+	gp_log (GP_LOG_ERROR, "context", "%s", str);
 
-	if (context->error_func) {
-		va_start (args, format);
-		context->error_func (context, format, args,
-				     context->error_func_data);
-		va_end (args);
-	}
+	if (context && context->error_func)
+		context->error_func (context, str, context->error_func_data);
+	free (str);
 }
 
 void
 gp_context_status (GPContext *context, const char *format, ...)
 {
 	va_list args;
+#ifdef HAVE_VA_COPY
+	va_list xargs;
+#endif
+	int strsize = 1000;
+	char *str;
+	int n;
 
-	/* Log the status message */
+	str = malloc(strsize);
+	if (!str) {
+		return;
+	}
 	va_start (args, format);
-	gp_logv (GP_LOG_DEBUG, "context", format, args);
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#endif
+	n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+	va_end (xargs);
+#endif
+	if (n+1>strsize) {
+		free (str);
+		str = malloc(n+1);
+		if (!str) { 
+			va_end (args);
+			return;
+		}
+		strsize = n+1;
+#ifdef HAVE_VA_COPY
+		va_copy (xargs, args);
+#endif
+		n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+		va_end (xargs);
+#endif
+	}
 	va_end (args);
 
-	if (!context)
-		return;
+	/* Log the status message */
+	gp_log (GP_LOG_DEBUG, "context", "%s", str);
 
-	if (context->status_func) {
-		va_start (args, format);
-		context->status_func (context, format, args,
-				      context->status_func_data);
-		va_end (args);
-	}
+	if (context && context->status_func)
+		context->status_func (context, str, context->status_func_data);
+	free (str);
 }
 
 /**
@@ -253,21 +347,50 @@ void
 gp_context_message (GPContext *context, const char *format, ...)
 {
 	va_list args;
+#ifdef HAVE_VA_COPY
+	va_list xargs;
+#endif
+	int strsize = 1000;
+	char *str;
+	int n;
 
-	/* Log the message */
+	str = malloc(strsize);
+	if (!str) {
+		va_end (xargs);
+		return;
+	}
 	va_start (args, format);
-	gp_logv (GP_LOG_DEBUG, "context", format, args);
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#endif
+	n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+	va_end (xargs);
+#endif
+	if (n+1>strsize) {
+		free (str);
+		str = malloc(n+1);
+		if (!str) { 
+			va_end (args);
+			return;
+		}
+		strsize = n+1;
+#ifdef HAVE_VA_COPY
+		va_copy (xargs, args);
+#endif
+		n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+		va_end (xargs);
+#endif
+	}
 	va_end (args);
 
-	if (!context)
-		return;
+	/* Log the message */
+	gp_log (GP_LOG_DEBUG, "context", "%s", str);
 
-	if (context->message_func) {
-		va_start (args, format);
-		context->message_func (context, format, args,
-				       context->message_func_data);
-		va_end (args);
-	}
+	if (context && context->message_func)
+		context->message_func (context, str, context->message_func_data);
+	free (str);
 }
 
 /**
@@ -288,18 +411,50 @@ gp_context_question (GPContext *context, const char *format, ...)
 {
 	GPContextFeedback feedback;
 	va_list args;
+#ifdef HAVE_VA_COPY
+	va_list xargs;
+#endif
+	int strsize = 1000;
+	char *str;
+	int n;
 
-	if (!context)
-		return (GP_CONTEXT_FEEDBACK_OK);
+	str = malloc(strsize);
+	if (!str) {
+		return GP_CONTEXT_FEEDBACK_OK;
+	}
+	va_start (args, format);
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#endif
+	n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+	va_end (xargs);
+#endif
+	if (n+1>strsize) {
+		free (str);
+		str = malloc(n+1);
+		if (!str) { 
+			va_end (args);
+			return GP_CONTEXT_FEEDBACK_OK;
+		}
+		strsize = n+1;
+#ifdef HAVE_VA_COPY
+		va_copy (xargs, args);
+#endif
+		n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+		va_end (xargs);
+#endif
+	}
+	va_end (args);
 
-	if (context->question_func) {
-		va_start (args, format);
-		feedback = context->question_func (context, format, args,
-						   context->question_func_data);
-		va_end (args);
-		return (feedback);
-	} else
-		return (GP_CONTEXT_FEEDBACK_OK);
+	feedback = GP_CONTEXT_FEEDBACK_OK;
+	if (context && context->question_func)
+		feedback = context->question_func (context, str, context->question_func_data);
+
+	free (str);
+
+	return feedback;
 }
 
 /**
