@@ -1,20 +1,23 @@
-/* jpeg.c
- * This code was written by Nathan Stenzel for gphoto
+/** \file
+ * \author This code was written by Nathan Stenzel for gphoto
  *
+ * \par License
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+ * \par
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
+ * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -22,7 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <gphoto2/gphoto2-library.h>
+#include <gphoto2/gphoto2-file.h>
 
 /* call example:nullpictureabort(picture,"Picture",0); */
 #define nullpointerabort(pointer,name,val) \
@@ -51,14 +54,17 @@ const char *JPEG_MARKERNAMES[] = {
 
 chunk *gpi_jpeg_chunk_new(int length)
 {
-chunk *mychunk;
-    printf("Entered gpi_jpeg_chunk_new\n");
-    mychunk =  (chunk *) malloc(sizeof(chunk));
-    mychunk->size=length;
+	chunk *mychunk;
+	printf("Entered gpi_jpeg_chunk_new\n");
+	mychunk =  (chunk *) malloc(sizeof(chunk));
+	if (mychunk==NULL) {
+		printf("Failed to allocate new chunk!\n");
+		return NULL;
+	}
+	mychunk->size=length;
 /*    printf("New chunk of size %i\n", mychunk->size); */
-    mychunk->data = (char *)malloc(length);
-    if (mychunk==NULL) printf("Failed to allocate new chunk!\n");
-    return mychunk;
+	mychunk->data = malloc(length);
+	return mychunk;
 }
 
 chunk *gpi_jpeg_chunk_new_filled(int length, char *data)
@@ -66,6 +72,8 @@ chunk *gpi_jpeg_chunk_new_filled(int length, char *data)
     chunk *mychunk;
     printf("Entered gpi_jpeg_chunk_new_filled\n");
     mychunk = gpi_jpeg_chunk_new(length);
+    if (!mychunk)
+	return NULL;
     printf("Filling the chunk data via chunk_new_filled\n");
     memcpy(mychunk->data, data, length);
     return mychunk;
@@ -120,9 +128,9 @@ char gpi_jpeg_findactivemarker(char *id, int *location, chunk *picture)
     return 0;
 }
 
-char *gpi_jpeg_markername(int c)
+char *gpi_jpeg_markername(unsigned int c)
 {
-    int x;
+    unsigned int x;
 /*    printf("searching for marker %X in list\n",c); */
 /*    printf("%i\n", sizeof(markers)); */
     for (x=0; x<countof(JPEG_MARKERS); x++)
@@ -157,9 +165,11 @@ void gpi_jpeg_add_marker(jpeg *myjpeg, chunk *picture, int start, int end)
     int length;
     nullpointerabortvoid(picture, "Picture");
     length=(int)(end-start+1);
-/*    printf("Add marker #%i starting from %i and ending at %i for a length of %i\n", myjpeg->count, start, end, length); */
+/*  printf("Add marker #%i starting from %i and ending at %i for a length of %i\n", myjpeg->count, start, end, length); */
     myjpeg->marker[myjpeg->count] = gpi_jpeg_chunk_new(length);
-/*    printf("Read length is: %i\n", myjpeg->marker[myjpeg->count].size); */
+    if (!myjpeg->marker[myjpeg->count])
+	return;
+/*  printf("Read length is: %i\n", myjpeg->marker[myjpeg->count].size); */
     memcpy(myjpeg->marker[myjpeg->count]->data, picture->data+start, length);
     gpi_jpeg_chunk_print(myjpeg->marker[myjpeg->count]);
     myjpeg->count++;
@@ -258,6 +268,7 @@ chunk *gpi_jpeg_makeSsSeAhAl(int huffset1, int huffset2, int huffset3)
     printf("About to call gpi_jpeg_chunk_new_filled\n");
     target= gpi_jpeg_chunk_new_filled(14, "\xFF\xDA\x00\x0C\x03\x01\x00\x02"
         "\x00\x03\x00\x00\x3F\x00");
+    if (!target) return NULL;
     target->data[6] = huffset1;
     target->data[8] = huffset2;
     target->data[10] = huffset3;
@@ -285,6 +296,7 @@ chunk *gpi_jpeg_make_quantization(jpeg_quantization_table *table, char number)
     chunk *temp;
     char x,y,z,c;
     temp = gpi_jpeg_chunk_new(5+64);
+    if (!temp) return NULL;
     memcpy(temp->data, "\xFF\xDB\x00\x43\x01", 5);
     temp->data[4]=number;
     for (c=z=0; z<8; z++)
@@ -356,7 +368,7 @@ char gpi_jpeg_write(CameraFile *file, const char *filename, jpeg *myjpeg)
     CHECK_RESULT (gp_file_set_name (file, filename));
     CHECK_RESULT (gp_file_set_mime_type(file, GP_MIME_JPEG));
     for (x=0; x<myjpeg->count; x++)
-        CHECK_RESULT (gp_file_append(file, myjpeg->marker[x]->data, myjpeg->marker[x]->size));
+        CHECK_RESULT (gp_file_append(file, (char*)myjpeg->marker[x]->data, myjpeg->marker[x]->size));
     return 1;
 }
 #endif
