@@ -1,26 +1,30 @@
-/* jpeg.h
- * This code was written by Nathan Stenzel for gphoto
+/** \file
+ * 
+ * \author This code was written by Nathan Stenzel for gphoto
  *
+ * \note
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+ * \note
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
  *
+ * \note
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
 #ifndef __GPHOTO2_JPEG_H__
 #define __GPHOTO2_JPEG_H__
 
-#include <gphoto2-file.h>
+#include <gphoto2/gphoto2-file.h>
 
 typedef enum {
     JPEG_START=0xD8,        JPEG_COMMENT=0xFE,      JPEG_APPO=0xE0,
@@ -47,7 +51,7 @@ void gpi_jpeg_chunk_print(chunk *mychunk);
 
 char  gpi_jpeg_findff(int *location, chunk *picture);
 char  gpi_jpeg_findactivemarker(char *id, int *location, chunk *picture);
-char *gpi_jpeg_markername(int c);
+char *gpi_jpeg_markername(unsigned int c);
 
 jpeg *gpi_jpeg_new        (void);
 void  gpi_jpeg_destroy    (jpeg *myjpeg);
