@@ -1,23 +1,28 @@
-/* gphoto2-port-log.c
+/** \file
  *
- * Copyright  2001 Lutz Mller <lutz@users.sf.net>
+ * \author Copyright 2001 Lutz Mller <lutz@users.sf.net>
  *
+ * \par License
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
+ * \par
  * This library is distributed in the hope that it will be useful, 
  * but WITHOUT ANY WARRANTY; without even the implied warranty of 
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details. 
  *
+ * \par
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 
+#define _BSD_SOURCE
+
 #include "config.h"
 #include <gphoto2/gphoto2-port-log.h>
 
@@ -43,28 +48,35 @@
 #endif
 
 #ifndef DISABLE_DEBUGGING
+/** 
+ * \brief Internal logging function entry.
+ * 
+ * Internal structure to remember the logging functions.
+ * Use gp_log_add_func() and gp_log_remove_func() to access it.
+ */
 typedef struct {
-	unsigned int id;
-	GPLogLevel   level;
-	GPLogFunc    func;
-	void        *data;
+	unsigned int id;	/**< Internal id */
+	GPLogLevel   level;	/**< Internal loglevel */
+	GPLogFunc    func;	/**< Internal function pointer to call */
+	void        *data;	/**< Private data supplied by caller */
 } LogFunc;
 
 static LogFunc *log_funcs = NULL;
 static unsigned int log_funcs_count = 0;
 
 /**
- * gp_log_add_func:
- * @level: a #GPLogLevel
- * @func: a #GPLogFunc
- * @data: data
+ * \brief Add a function to get logging information
+ *
+ * \param level the maximum level of logging it will get, up to and including the passed value
+ * \param func a #GPLogFunc
+ * \param data data
  *
  * Adds a log function that will be called for each log message that is flagged
- * with a log level that appears in given log @level. This function returns
+ * with a log level that appears in given log level. This function returns
  * an id that you can use for removing the log function again (using
  * #gp_log_remove_func).
  *
- * Return value: an id or a gphoto2 error code
+ * \return an id or a gphoto2 error code
  **/
 int
 gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
@@ -94,12 +106,12 @@ gp_log_add_func (GPLogLevel level, GPLogFunc func, void *data)
 }
 
 /**
- * gp_log_remove_func:
- * @id: an id (return value of #gp_log_add_func)
+ * \brief Remove a logging receiving function
+ * \param id an id (return value of #gp_log_add_func)
  *
- * Removes the log function with given @id.
+ * Removes the log function with given id.
  *
- * Return value: a gphoto2 error code
+ * \return a gphoto2 error code
  **/
 int
 gp_log_remove_func (int id)
@@ -113,31 +125,31 @@ gp_log_remove_func (int id)
 	return (GP_OK);
 }
 
-/*
+/**
  * Width of offset field in characters. Note that HEXDUMP_COMPLETE_LINE 
  * needs to be changed when this value is changed.
  */
 #define HEXDUMP_OFFSET_WIDTH 4
 
-/*
+/**
  * Distance between offset, hexdump and ascii blocks. Note that
  * HEXDUMP_COMPLETE_LINE needs to be changed when this value is changed.
  */
 #define HEXDUMP_BLOCK_DISTANCE 2
 
-/* Initial value for x "pointer" (for hexdump) */
+/** Initial value for x "pointer" (for hexdump) */
 #define HEXDUMP_INIT_X (HEXDUMP_OFFSET_WIDTH + HEXDUMP_BLOCK_DISTANCE)
 
-/* Initial value for y "pointer" (for ascii values) */
+/** Initial value for y "pointer" (for ascii values) */
 #define HEXDUMP_INIT_Y (HEXDUMP_INIT_X + 3 * 16 - 1 + HEXDUMP_BLOCK_DISTANCE)
 
-/* Used to switch to next line */
+/** Used to switch to next line */
 #define HEXDUMP_LINE_WIDTH (HEXDUMP_INIT_Y + 16)
 
-/* Used to put the '-' character in the middle of the hexdumps */
+/** Used to put the '-' character in the middle of the hexdumps */
 #define HEXDUMP_MIDDLE (HEXDUMP_INIT_X + 3 * 8 - 1)
 
-/*
+/**
  * We are lazy and do our typing only once. Please note that you have
  * to add/remove some lines when increasing/decreasing the values of 
  * HEXDUMP_BLOCK_DISTANCE and/or HEXDUMP_OFFSET_WIDTH.
@@ -156,22 +168,22 @@ gp_log_remove_func (int id)
         curline = curline + (HEXDUMP_LINE_WIDTH + 1);}
 
 /**
- * gp_log_data:
- * @domain: the domain
- * @data: the data to be logged
- * @size: the size of the @data
+ * \brief Log data
+ * \brief domain the domain
+ * \brief data the data to be logged
+ * \brief size the size of the data
  *
- * Takes the @data and creates a formatted hexdump string. If you would like
+ * Takes the data and creates a formatted hexdump string. If you would like
  * to log text messages, use #gp_log instead.
  **/
 void
 gp_log_data (const char *domain, const char *data, unsigned int size)
 {
-	const static char hexchars[16] = "0123456789abcdef";
+	static const char hexchars[16] = "0123456789abcdef";
 	char *curline, *result;
 	int x = HEXDUMP_INIT_X;
 	int y = HEXDUMP_INIT_Y;
-	int index;
+	unsigned int index;
 	unsigned char value;
 
 	if (!data) {
@@ -237,47 +249,73 @@ gp_log_data (const char *domain, const char *data, unsigned int size)
 #undef HEXDUMP_OFFSET_WIDTH
 
 /**
- * gp_logv:
- * @level: gphoto2 log level
- * @domain: the domain
- * @format: the format
- * @args: the va_list corresponding to @format
+ * \brief Log a debug or error message with va_list
+ * \param level gphoto2 log level
+ * \param domain the domain
+ * \param format the format
+ * \param args the va_list corresponding to format
  *
- * Logs a message at the given log @level. You would normally use this
+ * Logs a message at the given log level. You would normally use this
  * function to log as yet unformatted strings. 
  **/
 void
 gp_logv (GPLogLevel level, const char *domain, const char *format,
 	 va_list args)
 {
-	int i;
+	unsigned int i;
 #ifdef HAVE_VA_COPY
 	va_list xargs;
-#endif
-
-	for (i = 0; i < log_funcs_count; i++) {
-		if (log_funcs[i].level >= level) {
-#ifdef HAVE_VA_COPY
-			va_copy (xargs, args);
-			log_funcs[i].func (level, domain, format, xargs,
-					   log_funcs[i].data);
 #else
-			log_funcs[i].func (level, domain, format, args,
-					   log_funcs[i].data);
+#define xargs args
 #endif
+	int strsize = 1000;
+	char *str;
+	int n;
+
+	if (!log_funcs_count)
+		return;
+
+	str = malloc(strsize);
+	if (!str) return;
+#ifdef HAVE_VA_COPY
+	va_copy (xargs, args);
+#endif
+	n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+	va_end(xargs);
+#endif
+	if (n+1>strsize) {
+		free (str);
+		str = malloc(n+1);
+		if (!str) {
+			va_end(args);
+			return;
 		}
+		strsize = n+1;
+#ifdef HAVE_VA_COPY
+		va_copy (xargs, args);
+#endif
+		n = vsnprintf (str, strsize, format, xargs);
+#ifdef HAVE_VA_COPY
+		va_end(xargs);
+#endif
 	}
+	va_end(args);
+	for (i = 0; i < log_funcs_count; i++)
+		if (log_funcs[i].level >= level)
+			log_funcs[i].func (level, domain, str, log_funcs[i].data);
+	free (str);
 }
 
 /**
- * gp_log:
- * @level: gphoto2 log level
- * @domain: the domain
- * @format: the format
- * @...:
+ * \brief Log a debug or error message
+ * \param level gphoto2 log level
+ * \param domain the log domain
+ * \param format a printf style format string
+ * \param ... the variable argumentlist for above format string
  *
- * Logs a message at the given log @level. You would normally use this
- * function to log strings.
+ * Logs a message at the given log level. You would normally use this
+ * function to log general debug output in a printf way.
  **/
 void
 gp_log (GPLogLevel level, const char *domain, const char *format, ...)
@@ -345,20 +383,26 @@ gp_log (GPLogLevel level, const char *domain, const char *format, ...)
 
 #ifdef _GPHOTO2_INTERNAL_CODE
 
-void
+/** 
+ * \brief (Internal) translate a enumeration code to a string
+ */
+const char *
 gpi_enum_to_string(unsigned int _enum, 
-		   const StringFlagItem *map,
-		   string_item_func func, void *data)
+		   const StringFlagItem *map)
 {
 	int i;
 	for (i=0; map[i].str != NULL; i++) {
 		if (_enum == map[i].flag) {
-			func(map[i].str, data);
+			return(map[i].str);
 			break;
 		}
 	}
+	return NULL;
 }
 
+/** 
+ * \brief (Internal) translate a string to an enumeration code
+ */
 int
 gpi_string_to_enum(const char *str,
 		   unsigned int *result,
@@ -436,3 +480,11 @@ gpi_string_list_to_flags(const char *str[],
 }
 
 #endif /* _GPHOTO2_INTERNAL_CODE */
+
+
+/*
+ * Local Variables:
+ * c-file-style:"linux"
+ * indent-tabs-mode:t
+ * End:
+ */
