@@ -15,8 +15,9 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the GPIO Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   write to the 
+   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA  02110-1301  USA
  */
 
 #include "config.h"
@@ -31,7 +32,7 @@
 #include <string.h>
 
 /*#include <usb.h>*/
-#include "gphoto2-port.h"
+#include <gphoto2/gphoto2-port.h>
 
 
 int gp_port_usb_list(gp_port_info *list, int *count);
