@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA  02110-1301  USA
  */
 #include "config.h"
 
@@ -30,7 +30,14 @@
 #include <gphoto2/gphoto2-camera.h>
 #include <gphoto2/gphoto2-port-portability.h>
 
-#define CHECK(f) {int res = f; if (res < 0) {printf ("ERROR: %s\n", gp_result_as_string (res)); return (1);}}
+#define CHECK(f) \
+	do { \
+		int res = f; \
+		if (res < 0) { \
+			printf ("ERROR: %s\n", gp_result_as_string (res)); \
+			return (1); \
+		} \
+	} while (0)
 
 
 #ifdef __GNUC__
@@ -49,11 +56,8 @@ struct timeval glob_tv_zero = { 0, 0 };
 
 
 static void
-#ifdef __GNUC__
-		__attribute__((__format__(printf,3,0)))
-#endif
-debug_func (GPLogLevel level, const char *domain, const char *format,
-	    va_list args, void __unused__ *data)
+debug_func (GPLogLevel level, const char *domain, const char *str,
+	    void __unused__ *data)
 {
 	struct timeval tv;
 	long sec, usec;
@@ -62,9 +66,7 @@ debug_func (GPLogLevel level, const char *domain, const char *format,
 	sec = tv.tv_sec  - glob_tv_zero.tv_sec;
 	usec = tv.tv_usec - glob_tv_zero.tv_usec;
 	if (usec < 0) {sec--; usec += 1000000L;}
-	fprintf (stderr, "%li.%06li %s(%i): ", sec, usec, domain, level);
-	vfprintf (stderr, format, args);
-	fputc ('\n', stderr);
+	fprintf (stderr, "%li.%06li %s(%i): %s\n", sec, usec, domain, level, str);
 }
 
 
@@ -90,7 +92,7 @@ print_hline (void)
 
 /** C equivalent of basename(1) */
 static const char *
-basename (const char *pathname)
+path_basename (const char *pathname)
 {
 	char *result, *tmp;
 	/* remove path part from camlib name */
@@ -128,7 +130,7 @@ typedef enum {
 static OutputFormat format = FMT_HEADED_TEXT;
 
 
-// #define DEBUG_OUTPUT
+/* #define DEBUG_OUTPUT */
 
 
 /** Parse command line and set global variables. */
@@ -159,7 +161,7 @@ parse_command_line (const int argc, char *argv[])
 		} else if (strcmp(argv[i], "--format=camlibs") == 0) {
 			format = FMT_FLAT_CAMLIBS;
 		} else {
-			const char * const bn = basename(argv[0]);
+			const char * const bn = path_basename(argv[0]);
 			printf("Unknown command line parameter %d: \"%s\"\n",
 			       i, argv[i]);
 			printf("Sorry, no more help to give but the "
@@ -181,6 +183,7 @@ main (int argc, char *argv[])
 	int i;
 	int count;
 	const char *fmt_str = NULL;
+	char lastmodel[200];
 
 	parse_command_line (argc, argv);
 
@@ -204,7 +207,7 @@ main (int argc, char *argv[])
 		const char *camlib_env = getenv(CAMLIBDIR_ENV);
 		const char *camlibs = (camlib_env != NULL)?camlib_env:CAMLIBS;
 
-		printf("no camera drivers (camlibs) found in camlib dir:\n"
+		printf("No camera drivers (camlibs) found in camlib dir:\n"
 		       "    CAMLIBS='%s', default='%s', used=%s\n",
 		       camlib_env?camlib_env:"(null)", CAMLIBS,
 		       (camlib_env!=NULL)?"CAMLIBS":"default");
@@ -242,13 +245,20 @@ main (int argc, char *argv[])
 		break;
 	}
 
+	strcpy(lastmodel,"xxx");
 	/* For each camera in the list, add a text snippet to the 
 	 * output file. */
 	for (i = 0; i < count; i++) {
 		CameraAbilities abilities;
 		const char *camlib_basename;
 		CHECK (gp_abilities_list_get_abilities (al, i, &abilities));
-		camlib_basename = basename(abilities.library);
+		camlib_basename = path_basename(abilities.library);
+
+		if (!strcmp(lastmodel, abilities.model)) {
+			fprintf(stderr,"Duplicated model name in camera list: %s\n", lastmodel);
+			exit(1);
+		}
+		strcpy(lastmodel,abilities.model);
 
 		switch (format) {
 		case FMT_HEADED_TEXT:
